@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: config.h 1.176.1.6 2003/11/14 13:29:13 kls Exp $
+ * $Id: config.h 1.221 2005/06/18 10:29:25 kls Exp $
  */
 
 #ifndef __CONFIG_H
@@ -17,20 +17,23 @@
 #include <time.h>
 #include <unistd.h>
 #include "device.h"
+#include "i18n.h"
 #include "tools.h"
 
-#define VDRVERSION  "1.2.6"
-#define VDRVERSNUM   10206  // Version * 10000 + Major * 100 + Minor
+#define VDRVERSION  "1.3.27"
+#define VDRVERSNUM   10327  // Version * 10000 + Major * 100 + Minor
 
 #define MAXPRIORITY 99
 #define MAXLIFETIME 99
 
-#define MINOSDWIDTH  40
-#define MAXOSDWIDTH  56
-#define MINOSDHEIGHT 12
-#define MAXOSDHEIGHT 21
+#define MINOSDWIDTH  480
+#define MAXOSDWIDTH  672
+#define MINOSDHEIGHT 324
+#define MAXOSDHEIGHT 567
 
 #define MaxFileName 256
+#define MaxSkinName 16
+#define MaxThemeName 16
 
 class cCommand : public cListObject {
 private:
@@ -137,7 +140,7 @@ public:
   bool Save(void)
   {
     bool result = true;
-    T *l = (T *)First();
+    T *l = (T *)this->First();
     cSafeFile f(fileName);
     if (f.Open()) {
        while (l) {
@@ -182,7 +185,7 @@ public:
   cSetupLine(void);
   cSetupLine(const char *Name, const char *Value, const char *Plugin = NULL);
   virtual ~cSetupLine();
-  virtual bool operator< (const cListObject &ListObject);
+  virtual int Compare(const cListObject &ListObject) const;
   const char *Plugin(void) { return plugin; }
   const char *Name(void) { return name; }
   const char *Value(void) { return value; }
@@ -193,8 +196,8 @@ public:
 class cSetup : public cConfig<cSetupLine> {
   friend class cPlugin; // needs to be able to call Store()
 private:
-  void StoreCaCaps(const char *Name);
-  bool ParseCaCaps(const char *Value);
+  void StoreLanguages(const char *Name, int *Values);
+  bool ParseLanguages(const char *Value, int *Values);
   bool Parse(const char *Name, const char *Value);
   cSetupLine *Get(const char *Name, const char *Plugin = NULL);
   void Store(const char *Name, const char *Value, const char *Plugin = NULL, bool AllowMultiple = false);
@@ -203,9 +206,12 @@ public:
   // Also adjust cMenuSetup (menu.c) when adding parameters here!
   int __BeginData__;
   int OSDLanguage;
+  char OSDSkin[MaxSkinName];
+  char OSDTheme[MaxThemeName];
   int PrimaryDVB;
   int ShowInfoOnChSwitch;
   int MenuScrollPage;
+  int MenuScrollWrap;
   int MarkInstantRecord;
   char NameInstantRecord[MaxFileName];
   int InstantRecordTime;
@@ -214,10 +220,14 @@ public:
   int LnbFrequHi;
   int DiSEqC;
   int SetSystemTime;
+  int TimeSource;
   int TimeTransponder;
   int MarginStart, MarginStop;
+  int AudioLanguages[I18nNumLanguages + 1];
+  int EPGLanguages[I18nNumLanguages + 1];
   int EPGScanTimeout;
   int EPGBugfixLevel;
+  int EPGLinger;
   int SVDRPTimeout;
   int ZapTimeout;
   int SortTimers;
@@ -225,21 +235,27 @@ public:
   int DefaultPriority, DefaultLifetime;
   int PausePriority, PauseLifetime;
   int UseSubtitle;
+  int UseVps;
+  int VpsMargin;
   int RecordingDirs;
+  int VideoDisplayFormat;
   int VideoFormat;
-  int RecordDolbyDigital;
+  int UpdateChannels;
+  int UseDolbyDigital;
   int ChannelInfoPos;
-  int OSDwidth, OSDheight;
+  int ChannelInfoTime;
+  int OSDLeft, OSDTop, OSDWidth, OSDHeight;
   int OSDMessageTime;
+  int UseSmallFont;
   int MaxVideoFileSize;
   int SplitEditedFiles;
   int MinEventTimeout, MinUserInactivity;
   int MultiSpeedMode;
   int ShowReplayMode;
   int ResumeID;
-  int CaCaps[MAXDEVICES][MAXCACAPS];
   int CurrentChannel;
   int CurrentVolume;
+  int CurrentDolby;
   int __EndData__;
   cSetup(void);
   cSetup& operator= (const cSetup &s);
