@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: device.c 1.47.1.1 2003/11/07 13:16:12 kls Exp $
+ * $Id: device.c 1.103 2005/06/12 13:39:11 kls Exp $
  */
 
 #include "device.h"
@@ -13,17 +13,133 @@
 #include <sys/mman.h>
 #include "audio.h"
 #include "channels.h"
-#include "eit.h"
 #include "i18n.h"
 #include "player.h"
 #include "receiver.h"
 #include "status.h"
 #include "transfer.h"
 
+// --- cPesAssembler ---------------------------------------------------------
+
+class cPesAssembler {
+private:
+  uchar *data;
+  uint32_t tag;
+  int length;
+  int size;
+  bool Realloc(int Size);
+public:
+  cPesAssembler(void);
+  ~cPesAssembler();
+  int ExpectedLength(void) { return PacketSize(data); }
+  static int PacketSize(const uchar *data);
+  int Length(void) { return length; }
+  const uchar *Data(void) { return data; } // only valid if Length() >= 4
+  void Reset(void);
+  void Put(uchar c);
+  void Put(const uchar *Data, int Length);
+  bool IsPes(void);
+  };
+
+cPesAssembler::cPesAssembler(void)
+{
+  data = NULL;
+  size = 0;
+  Reset();
+}
+
+cPesAssembler::~cPesAssembler()
+{
+  free(data);
+}
+
+void cPesAssembler::Reset(void)
+{
+  tag = 0xFFFFFFFF;
+  length = 0;
+}
+
+bool cPesAssembler::Realloc(int Size)
+{
+  if (Size > size) {
+     size = max(Size, 2048);
+     data = (uchar *)realloc(data, size);
+     if (!data) {
+        esyslog("ERROR: can't allocate memory for PES assembler");
+        length = 0;
+        size = 0;
+        return false;
+        }
+     }
+  return true;
+}
+
+void cPesAssembler::Put(uchar c)
+{
+  if (length < 4) {
+     tag = (tag << 8) | c;
+     if ((tag & 0xFFFFFF00) == 0x00000100) {
+        if (Realloc(4)) {
+           *(uint32_t *)data = htonl(tag);
+           length = 4;
+           }
+        }
+     else if (length < 3)
+        length++;
+     }
+  else if (Realloc(length + 1))
+     data[length++] = c;
+}
+
+void cPesAssembler::Put(const uchar *Data, int Length)
+{
+  while (length < 4 && Length > 0) {
+        Put(*Data++);
+        Length--;
+        }
+  if (Length && Realloc(length + Length)) {
+     memcpy(data + length, Data, Length);
+     length += Length;
+     }
+}
+
+int cPesAssembler::PacketSize(const uchar *data)
+{
+  // we need atleast 6 bytes of data here !!!
+  switch (data[3]) {
+    default:
+    case 0x00 ... 0xB8: // video stream start codes
+    case 0xB9: // Program end
+    case 0xBC: // Programm stream map
+    case 0xF0 ... 0xFF: // reserved
+         return 6;
+
+    case 0xBA: // Pack header
+         if ((data[4] & 0xC0) == 0x40) // MPEG2
+            return 14;
+         // to be absolutely correct we would have to add the stuffing bytes
+         // as well, but at this point we only may have 6 bytes of data avail-
+         // able. So it's up to the higher level to resync...
+         //return 14 + (data[13] & 0x07); // add stuffing bytes
+         else // MPEG1
+            return 12;
+
+    case 0xBB: // System header
+    case 0xBD: // Private stream1
+    case 0xBE: // Padding stream
+    case 0xBF: // Private stream2 (navigation data)
+    case 0xC0 ... 0xCF: // all the rest (the real packets)
+    case 0xD0 ... 0xDF:
+    case 0xE0 ... 0xEF:
+         return 6 + data[4] * 256 + data[5];
+    }
+}
+
 // --- cDevice ---------------------------------------------------------------
 
 // The default priority for non-primary devices:
 #define DEFAULTPRIORITY  -1
+#define TUNER_LOCK_TIMEOUT 5000 // ms
 
 int cDevice::numDevices = 0;
 int cDevice::useDevice = 0;
@@ -36,6 +152,8 @@ cDevice::cDevice(void)
 {
   cardIndex = nextCardIndex++;
 
+  SetDescription("receiver on device %d", CardIndex() + 1);
+
   SetVideoFormat(Setup.VideoFormat);
 
   active = false;
@@ -43,16 +161,24 @@ cDevice::cDevice(void)
   mute = false;
   volume = Setup.CurrentVolume;
 
+  sectionHandler = NULL;
+  eitFilter = NULL;
+  patFilter = NULL;
+  sdtFilter = NULL;
+  nitFilter = NULL;
+
   ciHandler = NULL;
   player = NULL;
+  pesAssembler = new cPesAssembler;
+  ClrAvailableTracks();
+  currentAudioTrack = ttNone;
+  currentAudioTrackMissingCount = 0;
 
   for (int i = 0; i < MAXRECEIVERS; i++)
       receiver[i] = NULL;
 
-  if (numDevices < MAXDEVICES) {
+  if (numDevices < MAXDEVICES)
      device[numDevices++] = this;
-     SetCaCaps(cardIndex);
-     }
   else
      esyslog("ERROR: too many devices!");
 }
@@ -63,6 +189,12 @@ cDevice::~cDevice()
   for (int i = 0; i < MAXRECEIVERS; i++)
       Detach(receiver[i]);
   delete ciHandler;
+  delete nitFilter;
+  delete sdtFilter;
+  delete patFilter;
+  delete eitFilter;
+  delete sectionHandler;
+  delete pesAssembler;
 }
 
 void cDevice::SetUseDevice(int n)
@@ -105,6 +237,7 @@ bool cDevice::SetPrimaryDevice(int n)
         primaryDevice->MakePrimaryDevice(false);
      primaryDevice = device[n];
      primaryDevice->MakePrimaryDevice(true);
+     primaryDevice->SetVideoFormat(Setup.VideoFormat);
      return true;
      }
   esyslog("ERROR: invalid primary device number: %d", n + 1);
@@ -116,11 +249,6 @@ bool cDevice::HasDecoder(void) const
   return false;
 }
 
-cOsdBase *cDevice::NewOsd(int x, int y)
-{
-  return NULL;
-}
-
 cSpuDecoder *cDevice::GetSpuDecoder(void)
 {
   return NULL;
@@ -142,25 +270,27 @@ cDevice *cDevice::GetDevice(int Index)
 cDevice *cDevice::GetDevice(const cChannel *Channel, int Priority, bool *NeedsDetachReceivers)
 {
   cDevice *d = NULL;
-  int select = 7, pri;
+  int select = 8, pri;
 
   for (int i = 0; i < numDevices; i++) {
       bool ndr;
       if (device[i]->ProvidesChannel(Channel, Priority, &ndr)) { // this device is basicly able to do the job
          if (device[i]->Receiving() && !ndr)
             pri = 0; // receiving and allows additional receivers
-         else if (d && !device[i]->Receiving() && device[i]->ProvidesCa(Channel->Ca()) < d->ProvidesCa(Channel->Ca()))
+         else if (d && !device[i]->Receiving() && device[i]->ProvidesCa(Channel) < d->ProvidesCa(Channel))
             pri = 1; // free and fewer Ca's
+         else if (!device[i]->Receiving() && !device[i]->HasDecoder())
+            pri = 2; // free and not a full featured card
          else if (!device[i]->Receiving() && !device[i]->IsPrimaryDevice())
-            pri = 2; // free and not the primary device
+            pri = 3; // free and not the primary device
          else if (!device[i]->Receiving())
-            pri = 3; // free
+            pri = 4; // free
          else if (d && device[i]->Priority() < d->Priority())
-            pri = 4; // receiving but priority is lower
-         else if (d && device[i]->Priority() == d->Priority() && device[i]->ProvidesCa(Channel->Ca()) < d->ProvidesCa(Channel->Ca()))
-            pri = 5; // receiving with same priority but fewer Ca's
+            pri = 5; // receiving but priority is lower
+         else if (d && device[i]->Priority() == d->Priority() && device[i]->ProvidesCa(Channel) < d->ProvidesCa(Channel))
+            pri = 6; // receiving with same priority but fewer Ca's
          else
-            pri = 6; // all others
+            pri = 7; // all others
          if (pri < select) {
             select = pri;
             d = device[i];
@@ -188,23 +318,13 @@ cDevice *cDevice::GetDevice(const cChannel *Channel, int Priority, bool *NeedsDe
   return d;
 }
 
-void cDevice::SetCaCaps(int Index)
-{
-  for (int d = 0; d < numDevices; d++) {
-      if (Index < 0 || Index == device[d]->CardIndex()) {
-         for (int i = 0; i < MAXCACAPS; i++)
-             device[d]->caCaps[i] = Setup.CaCaps[device[d]->CardIndex()][i];
-         }
-      }
-}
-
 void cDevice::Shutdown(void)
 {
+  primaryDevice = NULL;
   for (int i = 0; i < numDevices; i++) {
       delete device[i];
       device[i] = NULL;
       }
-  primaryDevice = NULL;
 }
 
 bool cDevice::GrabImage(const char *FileName, bool Jpeg, int Quality, int SizeX, int SizeY)
@@ -212,6 +332,28 @@ bool cDevice::GrabImage(const char *FileName, bool Jpeg, int Quality, int SizeX,
   return false;
 }
 
+void cDevice::SetVideoDisplayFormat(eVideoDisplayFormat VideoDisplayFormat)
+{
+  cSpuDecoder *spuDecoder = GetSpuDecoder();
+  if (spuDecoder) {
+     if (Setup.VideoFormat)
+        spuDecoder->setScaleMode(cSpuDecoder::eSpuNormal);
+     else {
+        switch (VideoDisplayFormat) {
+               case vdfPanAndScan:
+                    spuDecoder->setScaleMode(cSpuDecoder::eSpuPanAndScan);
+                    break;
+               case vdfLetterBox:
+                    spuDecoder->setScaleMode(cSpuDecoder::eSpuLetterBox);
+                    break;
+               case vdfCenterCutOut:
+                    spuDecoder->setScaleMode(cSpuDecoder::eSpuNormal);
+                    break;
+               }
+        }
+     }
+}
+
 void cDevice::SetVideoFormat(bool VideoFormat16_9)
 {
 }
@@ -253,7 +395,13 @@ bool cDevice::AddPid(int Pid, ePidType PidType)
         if (++pidHandles[n].used == 2 && n <= ptTeletext) {
            // It's a special PID that may have to be switched into "tap" mode
            PRINTPIDS("A");
-           return SetPid(&pidHandles[n], n, true);
+           if (!SetPid(&pidHandles[n], n, true)) {
+              esyslog("ERROR: can't set PID %d on device %d", Pid, CardIndex() + 1);
+              if (PidType <= ptTeletext)
+                 DetachAll(Pid);
+              DelPid(Pid, PidType);
+              return false;
+              }
            }
         PRINTPIDS("a");
         return true;
@@ -266,13 +414,21 @@ bool cDevice::AddPid(int Pid, ePidType PidType)
         // The Pid is not yet in use and we have a free slot
         n = a;
         }
-     else
-        esyslog("ERROR: no free slot for PID %d", Pid);
+     else {
+        esyslog("ERROR: no free slot for PID %d on device %d", Pid, CardIndex() + 1);
+        return false;
+        }
      if (n >= 0) {
         pidHandles[n].pid = Pid;
         pidHandles[n].used = 1;
         PRINTPIDS("C");
-        return SetPid(&pidHandles[n], n, true);
+        if (!SetPid(&pidHandles[n], n, true)) {
+           esyslog("ERROR: can't set PID %d on device %d", Pid, CardIndex() + 1);
+           if (PidType <= ptTeletext)
+              DetachAll(Pid);
+           DelPid(Pid, PidType);
+           return false;
+           }
         }
      }
   return true;
@@ -311,11 +467,53 @@ bool cDevice::SetPid(cPidHandle *Handle, int Type, bool On)
   return false;
 }
 
+void cDevice::StartSectionHandler(void)
+{
+  if (!sectionHandler) {
+     sectionHandler = new cSectionHandler(this);
+     AttachFilter(eitFilter = new cEitFilter);
+     AttachFilter(patFilter = new cPatFilter);
+     AttachFilter(sdtFilter = new cSdtFilter(patFilter));
+     AttachFilter(nitFilter = new cNitFilter);
+     }
+}
+
+int cDevice::OpenFilter(u_short Pid, u_char Tid, u_char Mask)
+{
+  return -1;
+}
+
+void cDevice::AttachFilter(cFilter *Filter)
+{
+  if (sectionHandler)
+     sectionHandler->Attach(Filter);
+}
+
+void cDevice::Detach(cFilter *Filter)
+{
+  if (sectionHandler)
+     sectionHandler->Detach(Filter);
+}
+
 bool cDevice::ProvidesSource(int Source) const
 {
   return false;
 }
 
+bool cDevice::ProvidesTransponder(const cChannel *Channel) const
+{
+  return false;
+}
+
+bool cDevice::ProvidesTransponderExclusively(const cChannel *Channel) const
+{
+  for (int i = 0; i < numDevices; i++) {
+      if (device[i] && device[i] != this && device[i]->ProvidesTransponder(Channel))
+         return false;
+      }
+  return true;
+}
+
 bool cDevice::ProvidesChannel(const cChannel *Channel, int Priority, bool *NeedsDetachReceivers) const
 {
   return false;
@@ -328,11 +526,9 @@ bool cDevice::SwitchChannel(const cChannel *Channel, bool LiveView)
   for (int i = 3; i--;) {
       switch (SetChannel(Channel, LiveView)) {
         case scrOk:           return true;
-        case scrNotAvailable: if (Interface)
-                                 Interface->Error(tr("Channel not available!"));
+        case scrNotAvailable: Skins.Message(mtInfo, tr("Channel not available!"));
                               return false;
-        case scrNoTransfer:   if (Interface)
-                                 Interface->Error(tr("Can't start Transfer Mode!"));
+        case scrNoTransfer:   Skins.Message(mtError, tr("Can't start Transfer Mode!"));
                               return false;
         case scrFailed:       break; // loop will retry
         }
@@ -364,8 +560,8 @@ bool cDevice::SwitchChannel(int Direction)
         if (PrimaryDevice()->SwitchChannel(channel, true))
            result = true;
         }
-     else if (n != first && Interface)
-        Interface->Error(tr("Channel not available!"));
+     else if (n != first)
+        Skins.Message(mtError, tr("Channel not available!"));
      }
   return result;
 }
@@ -390,7 +586,7 @@ eSetChannelResult cDevice::SetChannel(const cChannel *Channel, bool LiveView)
      if (CaDevice && CanReplay()) {
         cStatus::MsgChannelSwitch(this, 0); // only report status if we are actually going to switch the channel
         if (CaDevice->SetChannel(Channel, false) == scrOk) // calling SetChannel() directly, not SwitchChannel()!
-           cControl::Launch(new cTransferControl(CaDevice, Channel->Vpid(), Channel->Apid1(), Channel->Apid2(), Channel->Dpid1(), Channel->Dpid2()));//XXX+
+           cControl::Launch(new cTransferControl(CaDevice, Channel->Vpid(), Channel->Apids(), Channel->Dpids(), Channel->Spids()));
         else
            Result = scrNoTransfer;
         }
@@ -398,15 +594,38 @@ eSetChannelResult cDevice::SetChannel(const cChannel *Channel, bool LiveView)
         Result = scrNotAvailable;
      }
   else {
+     Channels.Lock(false);
      cStatus::MsgChannelSwitch(this, 0); // only report status if we are actually going to switch the channel
-     if (!SetChannelDevice(Channel, LiveView))
+     // Stop section handling:
+     if (sectionHandler) {
+        sectionHandler->SetStatus(false);
+        sectionHandler->SetChannel(NULL);
+        }
+     if (SetChannelDevice(Channel, LiveView)) {
+        // Start section handling:
+        if (sectionHandler) {
+           sectionHandler->SetChannel(Channel);
+           sectionHandler->SetStatus(true);
+           }
+        }
+     else
         Result = scrFailed;
+     Channels.Unlock();
      }
 
   if (Result == scrOk) {
      if (LiveView && IsPrimaryDevice()) {
-        cSIProcessor::SetCurrentChannelID(Channel->GetChannelID());
         currentChannel = Channel->Number();
+        // Set the available audio tracks:
+        ClrAvailableTracks();
+        for (int i = 0; i < MAXAPIDS; i++)
+            SetAvailableTrack(ttAudio, i, Channel->Apid(i), Channel->Alang(i));
+        if (Setup.UseDolbyDigital) {
+           for (int i = 0; i < MAXDPIDS; i++)
+               SetAvailableTrack(ttDolby, i, Channel->Dpid(i), Channel->Dlang(i));
+           }
+        if (!NeedsTransferMode)
+           EnsureAudioTrack(true);
         }
      cStatus::MsgChannelSwitch(this, Channel->Number()); // only report status if channel switch successfull
      }
@@ -419,26 +638,34 @@ bool cDevice::SetChannelDevice(const cChannel *Channel, bool LiveView)
   return false;
 }
 
+bool cDevice::HasLock(int TimeoutMs)
+{
+  return true;
+}
+
 bool cDevice::HasProgramme(void)
 {
   return Replaying() || pidHandles[ptAudio].pid || pidHandles[ptVideo].pid;
 }
 
+int cDevice::GetAudioChannelDevice(void)
+{
+  return 0;
+}
+
+void cDevice::SetAudioChannelDevice(int AudioChannel)
+{
+}
+
 void cDevice::SetVolumeDevice(int Volume)
 {
 }
 
-int cDevice::NumAudioTracksDevice(void) const
+void cDevice::SetDigitalAudioDevice(bool On)
 {
-  return 0;
 }
 
-const char **cDevice::GetAudioTracksDevice(int *CurrentTrack) const
-{
-  return NULL;
-}
-
-void cDevice::SetAudioTrackDevice(int Index)
+void cDevice::SetAudioTrackDevice(eTrackType Type)
 {
 }
 
@@ -459,6 +686,18 @@ bool cDevice::ToggleMute(void)
   return mute;
 }
 
+int cDevice::GetAudioChannel(void)
+{
+  int c = GetAudioChannelDevice();
+  return (0 <= c && c <= 2) ? c : 0;
+}
+
+void cDevice::SetAudioChannel(int AudioChannel)
+{
+  if (0 <= AudioChannel && AudioChannel <= 2)
+     SetAudioChannelDevice(AudioChannel);
+}
+
 void cDevice::SetVolume(int Volume, bool Absolute)
 {
   volume = min(max(Absolute ? Volume : volume + Volume, 0), MAXVOLUME);
@@ -470,22 +709,101 @@ void cDevice::SetVolume(int Volume, bool Absolute)
      }
 }
 
+void cDevice::ClrAvailableTracks(bool DescriptionsOnly)
+{
+  if (DescriptionsOnly) {
+     for (int i = ttNone; i < ttMaxTrackTypes; i++)
+         *availableTracks[i].description = 0;
+     }
+  else {
+     memset(availableTracks, 0, sizeof(availableTracks));
+     pre_1_3_19_PrivateStream = false;
+     SetAudioChannel(0); // fall back to stereo
+     currentAudioTrackMissingCount = 0;
+     currentAudioTrack = ttNone;
+     }
+}
+
+bool cDevice::SetAvailableTrack(eTrackType Type, int Index, uint16_t Id, const char *Language, const char *Description)
+{
+  eTrackType t = eTrackType(Type + Index);
+  if (Type == ttAudio && IS_AUDIO_TRACK(t) ||
+      Type == ttDolby && IS_DOLBY_TRACK(t)) {
+     if (Language)
+        strn0cpy(availableTracks[t].language, Language, sizeof(availableTracks[t].language));
+     if (Description)
+        strn0cpy(availableTracks[t].description, Description, sizeof(availableTracks[t].description));
+     if (Id) {
+        availableTracks[t].id = Id; // setting 'id' last to avoid the need for extensive locking
+        int numAudioTracks = NumAudioTracks();
+        if (!availableTracks[currentAudioTrack].id && numAudioTracks && currentAudioTrackMissingCount++ > numAudioTracks * 10)
+           EnsureAudioTrack();
+        else if (t == currentAudioTrack)
+           currentAudioTrackMissingCount = 0;
+        }
+     return true;
+     }
+  else
+     esyslog("ERROR: SetAvailableTrack called with invalid Type/Index (%d/%d)", Type, Index);
+  return false;
+}
+
+const tTrackId *cDevice::GetTrack(eTrackType Type)
+{
+  return (ttNone < Type && Type < ttMaxTrackTypes) ? &availableTracks[Type] : NULL;
+}
+
 int cDevice::NumAudioTracks(void) const
 {
-  return player ? player->NumAudioTracks() : NumAudioTracksDevice();
+  int n = 0;
+  for (int i = ttAudioFirst; i <= ttDolbyLast; i++) {
+      if (availableTracks[i].id)
+         n++;
+      }
+  return n;
 }
 
-const char **cDevice::GetAudioTracks(int *CurrentTrack) const
+bool cDevice::SetCurrentAudioTrack(eTrackType Type)
 {
-  return player ? player->GetAudioTracks(CurrentTrack) : GetAudioTracksDevice(CurrentTrack);
+  if (ttNone < Type && Type < ttDolbyLast) {
+     if (IS_DOLBY_TRACK(Type))
+        SetDigitalAudioDevice(true);
+     currentAudioTrack = Type;
+     if (player)
+        player->SetAudioTrack(currentAudioTrack, GetTrack(currentAudioTrack));
+     else
+        SetAudioTrackDevice(currentAudioTrack);
+     if (IS_AUDIO_TRACK(Type))
+        SetDigitalAudioDevice(false);
+     return true;
+     }
+  return false;
 }
 
-void cDevice::SetAudioTrack(int Index)
+void cDevice::EnsureAudioTrack(bool Force)
 {
-  if (player)
-     player->SetAudioTrack(Index);
-  else
-     SetAudioTrackDevice(Index);
+  if (Force || !availableTracks[currentAudioTrack].id) {
+     eTrackType PreferredTrack = ttAudioFirst;
+     int LanguagePreference = -1;
+     int StartCheck = Setup.CurrentDolby ? ttDolbyFirst : ttAudioFirst;
+     int EndCheck = ttDolbyLast;
+     for (int i = StartCheck; i <= EndCheck; i++) {
+         const tTrackId *TrackId = GetTrack(eTrackType(i));
+         if (TrackId && TrackId->id && I18nIsPreferredLanguage(Setup.AudioLanguages, I18nLanguageIndex(TrackId->language), LanguagePreference))
+            PreferredTrack = eTrackType(i);
+         if (Setup.CurrentDolby && i == ttDolbyLast) {
+            i = ttAudioFirst - 1;
+            EndCheck = ttAudioLast;
+            }
+         }
+     // Make sure we're set to an available audio track:
+     const tTrackId *Track = GetTrack(GetCurrentAudioTrack());
+     if (Force || !Track || !Track->id || PreferredTrack != GetCurrentAudioTrack()) {
+        if (!Force) // only log this for automatic changes
+           dsyslog("setting audio track to %d", PreferredTrack);
+        SetCurrentAudioTrack(PreferredTrack);
+        }
+     }
 }
 
 bool cDevice::CanReplay(void) const
@@ -541,9 +859,12 @@ bool cDevice::AttachPlayer(cPlayer *Player)
   if (CanReplay()) {
      if (player)
         Detach(player);
+     if (!dynamic_cast<cTransfer *>(Player))
+        ClrAvailableTracks();
+     pesAssembler->Reset();
      player = Player;
-     player->device = this;
      SetPlayMode(player->playMode);
+     player->device = this;
      player->Activate(true);
      return true;
      }
@@ -557,6 +878,7 @@ void cDevice::Detach(cPlayer *Player)
      player->device = NULL;
      player = NULL;
      SetPlayMode(pmNone);
+     SetVideoDisplayFormat(eVideoDisplayFormat(Setup.VideoDisplayFormat));
      Audios.ClearAudio();
      }
 }
@@ -575,14 +897,155 @@ bool cDevice::Poll(cPoller &Poller, int TimeoutMs)
   return false;
 }
 
+bool cDevice::Flush(int TimeoutMs)
+{
+  return true;
+}
+
 int cDevice::PlayVideo(const uchar *Data, int Length)
 {
   return -1;
 }
 
-void cDevice::PlayAudio(const uchar *Data, int Length)
+int cDevice::PlayAudio(const uchar *Data, int Length)
 {
-  Audios.PlayAudio(Data, Length);
+  return -1;
+}
+
+int cDevice::PlayPesPacket(const uchar *Data, int Length, bool VideoOnly)
+{
+  bool FirstLoop = true;
+  uchar c = Data[3];
+  const uchar *Start = Data;
+  const uchar *End = Start + Length;
+  while (Start < End) {
+        int d = End - Start;
+        int w = d;
+        switch (c) {
+          case 0xBE:          // padding stream, needed for MPEG1
+          case 0xE0 ... 0xEF: // video
+               w = PlayVideo(Start, d);
+               break;
+          case 0xC0 ... 0xDF: // audio
+               SetAvailableTrack(ttAudio, c - 0xC0, c);
+               if (!VideoOnly && c == availableTracks[currentAudioTrack].id) {
+                  w = PlayAudio(Start, d);
+                  if (FirstLoop)
+                     Audios.PlayAudio(Data, Length, c);
+                  }
+               break;
+          case 0xBD: { // private stream 1
+               int PayloadOffset = Data[8] + 9;
+               uchar SubStreamId = Data[PayloadOffset];
+               uchar SubStreamType = SubStreamId & 0xF0;
+               uchar SubStreamIndex = SubStreamId & 0x1F;
+        
+               // Compatibility mode for old VDR recordings, where 0xBD was only AC3:
+pre_1_3_19_PrivateStreamDeteced:
+               if (pre_1_3_19_PrivateStream) {
+                  SubStreamId = c;
+                  SubStreamType = 0x80;
+                  SubStreamIndex = 0;
+                  }
+               switch (SubStreamType) {
+                 case 0x20: // SPU
+                 case 0x30: // SPU
+                      break;
+                 case 0x80: // AC3 & DTS
+                      if (Setup.UseDolbyDigital) {
+                         SetAvailableTrack(ttDolby, SubStreamIndex, SubStreamId);
+                         if (!VideoOnly && SubStreamId == availableTracks[currentAudioTrack].id) {
+                            w = PlayAudio(Start, d);
+                            if (FirstLoop)
+                               Audios.PlayAudio(Data, Length, SubStreamId);
+                            }
+                         }
+                      break;
+                 case 0xA0: // LPCM
+                      SetAvailableTrack(ttAudio, SubStreamIndex, SubStreamId);
+                      if (!VideoOnly && SubStreamId == availableTracks[currentAudioTrack].id) {
+                         w = PlayAudio(Start, d);
+                         if (FirstLoop)
+                            Audios.PlayAudio(Data, Length, SubStreamId);
+                         }
+                      break;
+                 default:
+                      // Compatibility mode for old VDR recordings, where 0xBD was only AC3:
+                      if (!pre_1_3_19_PrivateStream) {
+                         dsyslog("switching to pre 1.3.19 Dolby Digital compatibility mode");
+                         ClrAvailableTracks();
+                         pre_1_3_19_PrivateStream = true;
+                         goto pre_1_3_19_PrivateStreamDeteced;
+                         }
+                 }
+               }
+               break;
+          default:
+               ;//esyslog("ERROR: unexpected packet id %02X", c);
+          }
+        if (w > 0)
+           Start += w;
+        else {
+           if (Start != Data)
+              esyslog("ERROR: incomplete PES packet write!");
+           return Start == Data ? w : Start - Data;
+           }
+        FirstLoop = false;
+        }
+  return Length;
+}
+
+int cDevice::PlayPes(const uchar *Data, int Length, bool VideoOnly)
+{
+  if (!Data) {
+     pesAssembler->Reset();
+     return 0;
+     }
+  int Result = 0;
+  if (pesAssembler->Length()) {
+     // Make sure we have a complete PES header:
+     while (pesAssembler->Length() < 6 && Length > 0) {
+           pesAssembler->Put(*Data++);
+           Length--;
+           Result++;
+           }
+     if (pesAssembler->Length() < 6)
+        return Result; // Still no complete PES header - wait for more
+     int l = pesAssembler->ExpectedLength();
+     int Rest = min(l - pesAssembler->Length(), Length);
+     pesAssembler->Put(Data, Rest);
+     Data += Rest;
+     Length -= Rest;
+     Result += Rest;
+     if (pesAssembler->Length() < l)
+        return Result; // Still no complete PES packet - wait for more
+     // Now pesAssembler contains one complete PES packet.
+     int w = PlayPesPacket(pesAssembler->Data(), pesAssembler->Length(), VideoOnly);
+     if (w > 0)
+        pesAssembler->Reset();
+     return Result > 0 ? Result : w < 0 ? w : 0;
+     }
+  int i = 0;
+  while (i <= Length - 6) {
+        if (Data[i] == 0x00 && Data[i + 1] == 0x00 && Data[i + 2] == 0x01) {
+           int l = cPesAssembler::PacketSize(&Data[i]);
+           if (i + l > Length) {
+              // Store incomplete PES packet for later completion:
+              pesAssembler->Put(Data + i, Length - i);
+              return Length;
+              }
+           int w = PlayPesPacket(Data + i, l, VideoOnly);
+           if (w > 0)
+              i += l;
+           else
+              return i == 0 ? w : i;
+           }
+        else
+           i++;
+        }
+  if (i < Length)
+     pesAssembler->Put(Data + i, Length - i);
+  return Length;
 }
 
 int cDevice::Ca(void) const
@@ -608,6 +1071,7 @@ int cDevice::Priority(void) const
 int cDevice::CanShift(int Ca, int Priority, int UsedCards) const
 {
   return -1;//XXX+ too complex with multiple recordings per device
+  /*XXX
   // Test whether a receiver on this device can be shifted to another one
   // in order to perform a new receiving with the given Ca and Priority on this device:
   int ShiftLevel = -1; // default means this device can't be shifted
@@ -638,25 +1102,17 @@ int cDevice::CanShift(int Ca, int Priority, int UsedCards) const
   else if (Priority > this->Priority())
      ShiftLevel = 0; // no shifting necessary, this device can do the job
   return ShiftLevel;
+  XXX*/
 }
 
-int cDevice::ProvidesCa(int Ca) const
+int cDevice::ProvidesCa(const cChannel *Channel) const
 {
+  int Ca = Channel->Ca();
   if (Ca == CardIndex() + 1)
      return 1; // exactly _this_ card was requested
   if (Ca && Ca <= MAXDEVICES)
      return 0; // a specific card was requested, but not _this_ one
-  int result = Ca ? 0 : 1; // by default every card can provide FTA
-  int others = Ca ? 1 : 0;
-  for (int i = 0; i < MAXCACAPS; i++) {
-      if (caCaps[i]) {
-         if (caCaps[i] == Ca)
-            result = 1;
-         else
-            others++;
-         }
-      }
-  return result ? result + others : 0;
+  return !Ca; // by default every card can provide FTA
 }
 
 bool cDevice::Receiving(bool CheckAny) const
@@ -670,10 +1126,7 @@ bool cDevice::Receiving(bool CheckAny) const
 
 void cDevice::Action(void)
 {
-  dsyslog("receiver thread started on device %d (pid=%d)", CardIndex() + 1, getpid());
-
-  if (OpenDvr()) {
-     active = true;
+  if (active && OpenDvr()) {
      for (; active;) {
          // Read data from the DVR device:
          uchar *b = NULL;
@@ -694,8 +1147,6 @@ void cDevice::Action(void)
          }
      CloseDvr();
      }
-
-  dsyslog("receiver thread ended on device %d (pid=%d)", CardIndex() + 1, getpid());
 }
 
 bool cDevice::OpenDvr(void)
@@ -718,16 +1169,29 @@ bool cDevice::AttachReceiver(cReceiver *Receiver)
      return false;
   if (Receiver->device == this)
      return true;
+  if (!HasLock(TUNER_LOCK_TIMEOUT)) {
+     esyslog("ERROR: device %d has no lock, can't attach receiver!", CardIndex() + 1);
+     return false;
+     }
+  cMutexLock MutexLock(&mutexReceiver);
   for (int i = 0; i < MAXRECEIVERS; i++) {
       if (!receiver[i]) {
-         for (int n = 0; n < MAXRECEIVEPIDS; n++)
-             AddPid(Receiver->pids[n]);//XXX+ retval!
+         for (int n = 0; n < Receiver->numPids; n++) {
+             if (!AddPid(Receiver->pids[n])) {
+                for ( ; n-- > 0; )
+                    DelPid(Receiver->pids[n]);
+                return false;
+                }
+             }
          Receiver->Activate(true);
          Lock();
          Receiver->device = this;
          receiver[i] = Receiver;
          Unlock();
-         Start();
+         if (!active) {
+            active = true;
+            Start();
+            }
          return true;
          }
       }
@@ -740,6 +1204,7 @@ void cDevice::Detach(cReceiver *Receiver)
   if (!Receiver || Receiver->device != this)
      return;
   bool receiversLeft = false;
+  cMutexLock MutexLock(&mutexReceiver);
   for (int i = 0; i < MAXRECEIVERS; i++) {
       if (receiver[i] == Receiver) {
          Receiver->Activate(false);
@@ -747,7 +1212,7 @@ void cDevice::Detach(cReceiver *Receiver)
          receiver[i] = NULL;
          Receiver->device = NULL;
          Unlock();
-         for (int n = 0; n < MAXRECEIVEPIDS; n++)
+         for (int n = 0; n < Receiver->numPids; n++)
              DelPid(Receiver->pids[n]);
          }
       else if (receiver[i])
@@ -759,83 +1224,82 @@ void cDevice::Detach(cReceiver *Receiver)
      }
 }
 
+void cDevice::DetachAll(int Pid)
+{
+  if (Pid) {
+     cMutexLock MutexLock(&mutexReceiver);
+     for (int i = 0; i < MAXRECEIVERS; i++) {
+         cReceiver *Receiver = receiver[i];
+         if (Receiver && Receiver->WantsPid(Pid))
+            Detach(Receiver);
+         }
+     }
+}
+
 // --- cTSBuffer -------------------------------------------------------------
 
 cTSBuffer::cTSBuffer(int File, int Size, int CardIndex)
 {
+  SetDescription("TS buffer on device %d", CardIndex);
   f = File;
-  size = Size / TS_SIZE * TS_SIZE;
   cardIndex = CardIndex;
-  tsRead = tsWrite = 0;
-  buf = (f >= 0 && size >= TS_SIZE) ? MALLOC(uchar, size + TS_SIZE) : NULL;
-  // the '+ TS_SIZE' allocates some extra space for handling packets that got split by a buffer roll-over
-  firstRead = true;
+  delivered = false;
+  ringBuffer = new cRingBufferLinear(Size, TS_SIZE, true, "TS");
+  ringBuffer->SetTimeouts(100, 100);
+  active = true;
+  Start();
 }
 
 cTSBuffer::~cTSBuffer()
 {
-  free(buf);
+  active = false;
+  Cancel(3);
+  delete ringBuffer;
 }
 
-int cTSBuffer::Read(void)
+void cTSBuffer::Action(void)
 {
-  if (buf) {
-     cPoller Poller(f, false);
-     bool repeat;
-     int total = 0;
-     do {
-        repeat = false;
-        if (firstRead || Used() > TS_SIZE || Poller.Poll(100)) { // only wait if there's not enough data in the buffer
-           firstRead = false;
-           if (tsRead == tsWrite)
-              tsRead = tsWrite = 0; // keep the maximum buffer space available
-           if (tsWrite >= size && tsRead > 0)
-              tsWrite = 0;
-           int free = tsRead <= tsWrite ? size - tsWrite : tsRead - tsWrite - 1;
-           if (free > 0) {
-              int r = read(f, buf + tsWrite, free);
-              if (r > 0) {
-                 total += r;
-                 tsWrite += r;
-                 if (tsWrite >= size && tsRead > 0) {
-                    tsWrite = 0;
-                    repeat = true; // read again after a boundary roll-over
-                    }
-                 }
-              }
-           }
-        } while (repeat);
-     return total;
+  if (ringBuffer) {
+     bool firstRead = true;
+     cPoller Poller(f);
+     for (; active;) {
+         if (firstRead || Poller.Poll(100)) {
+            firstRead = false;
+            int r = ringBuffer->Read(f);
+            if (r < 0 && FATALERRNO) {
+               if (errno == EOVERFLOW)
+                  esyslog("ERROR: driver buffer overflow on device %d", cardIndex);
+               else {
+                  LOG_ERROR;
+                  break;
+                  }
+               }
+            }
+         }
      }
-  return -1;
 }
 
 uchar *cTSBuffer::Get(void)
 {
-  if (Used() >= TS_SIZE) {
-     uchar *p = buf + tsRead;
+  int Count = 0;
+  if (delivered) {
+     ringBuffer->Del(TS_SIZE);
+     delivered = false;
+     }
+  uchar *p = ringBuffer->Get(Count);
+  if (p && Count >= TS_SIZE) {
      if (*p != TS_SYNC_BYTE) {
-        esyslog("ERROR: not sync'ed to TS packet on device %d", cardIndex);
-        int tsMax = tsRead < tsWrite ? tsWrite : size;
-        for (int i = tsRead; i < tsMax; i++) {
-            if (buf[i] == TS_SYNC_BYTE) {
-               esyslog("ERROR: skipped %d bytes to sync on TS packet on device %d", i - tsRead, cardIndex);
-               tsRead = i;
-               return NULL;
+        for (int i = 1; i < Count; i++) {
+            if (p[i] == TS_SYNC_BYTE) {
+               Count = i;
+               break;
                }
             }
-        if ((tsRead = tsMax) >= size)
-           tsRead = 0;
+        ringBuffer->Del(Count);
+        esyslog("ERROR: skipped %d bytes to sync on TS packet on device %d", Count, cardIndex);
         return NULL;
         }
-     if (tsRead + TS_SIZE > size) {
-        // the packet rolled over the buffer boundary, so let's fetch the rest from the beginning (which MUST be there, since Used() >= TS_SIZE)
-        int rest = TS_SIZE - (size - tsRead);
-        memcpy(buf + size, buf, rest);
-        tsRead = rest;
-        }
-     else if ((tsRead += TS_SIZE) >= size)
-        tsRead = 0;
+     delivered = true;
      return p;
      }
   return NULL;
