@@ -4,19 +4,26 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: device.h 1.35 2003/11/07 13:15:45 kls Exp $
+ * $Id: device.h 1.59 2005/06/12 13:35:47 kls Exp $
  */
 
 #ifndef __DEVICE_H
 #define __DEVICE_H
 
 #include "ci.h"
+#include "eit.h"
+#include "filter.h"
+#include "nit.h"
+#include "pat.h"
+#include "ringbuffer.h"
+#include "sdt.h"
+#include "sections.h"
+#include "spu.h"
 #include "thread.h"
 #include "tools.h"
 
 #define MAXDEVICES         16 // the maximum number of devices in the system
-#define MAXCACAPS          16 // the maximum number of different CA values per device
-#define MAXPIDHANDLES      16 // the maximum number of different PIDs per device
+#define MAXPIDHANDLES      64 // the maximum number of different PIDs per device
 #define MAXRECEIVERS       16 // the maximum number of receivers per device
 #define MAXVOLUME         255
 #define VOLUMEDELTA         5 // used to increase/decrease the volume
@@ -31,6 +38,7 @@ enum ePlayMode { pmNone,           // audio/video from decoder
                  pmAudioVideo,     // audio/video from player
                  pmAudioOnly,      // audio only from player, video from decoder
                  pmAudioOnlyBlack, // audio only from player, no video (black screen)
+                 pmVideoOnly,      // video only from player, audio from decoder
                  pmExtern_THIS_SHOULD_BE_AVOIDED
                  // external player (e.g. MPlayer), release the device
                  // WARNING: USE THIS MODE ONLY AS A LAST RESORT, IF YOU
@@ -49,11 +57,39 @@ enum eVideoSystem { vsPAL,
                     vsNTSC
                   };
 
-class cOsdBase;
+enum eVideoDisplayFormat { vdfPanAndScan,
+                           vdfLetterBox,
+                           vdfCenterCutOut
+                         };
+
+enum eTrackType { ttNone,
+                  ttAudio,
+                  ttAudioFirst = ttAudio,
+                  ttAudioLast  = ttAudioFirst + 31, // MAXAPIDS - 1
+                  ttDolby,
+                  ttDolbyFirst = ttDolby,
+                  ttDolbyLast  = ttDolbyFirst + 15, // MAXDPIDS - 1
+                  /* future...
+                  ttSubtitle,
+                  ttSubtitleFirst = ttSubtitle,
+                  ttSubtitleLast  = ttSubtitleFirst + 7, // MAXSPIDS - 1
+                  */
+                  ttMaxTrackTypes
+                };
+
+#define IS_AUDIO_TRACK(t) (ttAudioFirst <= (t) && (t) <= ttAudioLast)
+#define IS_DOLBY_TRACK(t) (ttDolbyFirst <= (t) && (t) <= ttDolbyLast)
+
+struct tTrackId {
+  uint16_t id;          // The PES packet id or the PID.
+  char language[8];     // something like either "eng" or "deu/eng"
+  char description[32]; // something like "Dolby Digital 5.1"
+  };
+
 class cChannel;
 class cPlayer;
 class cReceiver;
-class cSpuDecoder;
+class cPesAssembler;
 
 /// The cDevice class is the base from which actual devices can be derived.
 
@@ -91,16 +127,12 @@ public:
          ///< given Priority.
          ///< See ProvidesChannel() for more information on how
          ///< priorities are handled, and the meaning of NeedsDetachReceivers.
-  static void SetCaCaps(int Index = -1);
-         ///< Sets the CaCaps of the given device according to the Setup data.
-         ///< By default the CaCaps of all devices are set.
   static void Shutdown(void);
          ///< Closes down all devices.
          ///< Must be called at the end of the program.
 private:
   static int nextCardIndex;
   int cardIndex;
-  int caCaps[MAXCACAPS];
 protected:
   cDevice(void);
   virtual ~cDevice();
@@ -127,7 +159,8 @@ public:
          ///< Returns the card index of this device (0 ... MAXDEVICES - 1).
   int DeviceNumber(void) const;
          ///< Returns the number of this device (0 ... MAXDEVICES - 1).
-  int ProvidesCa(int Ca) const;
+  virtual int ProvidesCa(const cChannel *Channel) const;//XXX PLUGINS.html!!!
+         //XXX describe changed functionality!!!
          ///< Checks whether this device provides the given value in its
          ///< caCaps. Returns 0 if the value is not provided, 1 if only this
          ///< value is provided, and > 1 if this and other values are provided.
@@ -137,15 +170,9 @@ public:
   virtual bool HasDecoder(void) const;
          ///< Tells whether this device has an MPEG decoder.
 
-// OSD facilities
+// SPU facilities
 
 public:
-  virtual cOsdBase *NewOsd(int x, int y);
-         ///< Creates a new cOsdBase object that can be used by the cOsd class
-         ///< to display information on the screen, with the upper left corner
-         ///< of the OSD at the given coordinates. If a derived cDevice doesn't
-         ///< implement this function, NULL will be returned by default (which
-         ///< means the device has no OSD capabilities).
   virtual cSpuDecoder *GetSpuDecoder(void);
          ///< Returns a pointer to the device's SPU decoder (or NULL, if this
          ///< device doesn't have an SPU decoder).
@@ -157,6 +184,11 @@ protected:
 public:
   virtual bool ProvidesSource(int Source) const;
          ///< Returns true if this device can provide the given source.
+  virtual bool ProvidesTransponder(const cChannel *Channel) const;
+         ///< XXX -> PLUGINS.html!
+  virtual bool ProvidesTransponderExclusively(const cChannel *Channel) const;
+         ///< Returns true if this is the only device that is able to provide
+         ///< the given channel's transponder.
   virtual bool ProvidesChannel(const cChannel *Channel, int Priority = -1, bool *NeedsDetachReceivers = NULL) const;
          ///< Returns true if this device can provide the given channel.
          ///< In case the device has cReceivers attached to it or it is the primary
@@ -188,6 +220,12 @@ protected:
 public:
   static int CurrentChannel(void) { return primaryDevice ? currentChannel : 0; }
          ///< Returns the number of the current channel on the primary device.
+  virtual bool HasLock(int TimeoutMs = 0);//XXX PLUGINS.html
+         ///< Returns true if the device has a lock on the requested transponder.
+         ///< Default is true, a specific device implementation may return false
+         ///< to indicate that it is not ready yet.
+         ///< If TimeoutMs is not zero, waits for the given number of milliseconds
+         ///< before returning false.
   virtual bool HasProgramme(void);
          ///< Returns true if the device is currently showing any programme to
          ///< the user, either through replaying or live.
@@ -222,6 +260,28 @@ protected:
          ///< Type indicates some special types of PIDs, which the device may
          ///< need to set in a specific way.
 
+// Section filter facilities
+
+private:
+  cSectionHandler *sectionHandler;
+  cEitFilter *eitFilter;
+  cPatFilter *patFilter;
+  cSdtFilter *sdtFilter;
+  cNitFilter *nitFilter;
+protected:
+  void StartSectionHandler(void);
+       ///< A derived device that provides section data must call
+       ///< this function to actually set up the section handler.
+public:
+  virtual int OpenFilter(u_short Pid, u_char Tid, u_char Mask);
+       ///< Opens a file handle for the given filter data.
+       ///< A derived device that provides section data must
+       ///< implement this function.
+  void AttachFilter(cFilter *Filter);
+       ///< Attaches the given filter to this device.
+  void Detach(cFilter *Filter);
+       ///< Detaches the given filter from this device.
+
 // Common Interface facilities:
 
 protected:
@@ -249,6 +309,10 @@ public:
 // Video format facilities
 
 public:
+  virtual void SetVideoDisplayFormat(eVideoDisplayFormat VideoDisplayFormat);
+         ///< Sets the video display format to the given one (only useful
+         ///< if this device has an MPEG decoder).
+         ///< A derived class must first call the base class function!
   virtual void SetVideoFormat(bool VideoFormat16_9);
          ///< Sets the output video format to either 16:9 or 4:3 (only useful
          ///< if this device has an MPEG decoder).
@@ -256,69 +320,102 @@ public:
          ///< Returns the video system of the currently displayed material
          ///< (default is PAL).
 
+// Track facilities
+
+private:
+  tTrackId availableTracks[ttMaxTrackTypes];
+  eTrackType currentAudioTrack;
+  int currentAudioTrackMissingCount;
+  bool pre_1_3_19_PrivateStream;
+protected:
+  virtual void SetAudioTrackDevice(eTrackType Type);
+       ///< Sets the current audio track to the given value.
+public:
+  void ClrAvailableTracks(bool DescriptionsOnly = false);
+  bool SetAvailableTrack(eTrackType Type, int Index, uint16_t Id, const char *Language = NULL, const char *Description = NULL);
+       ///< Sets the track of the given Type and Index to the given values.
+       ///< Type must be one of the basic eTrackType values, like ttAudio or ttDolby.
+       ///< Index tells which track of the given basic type is meant.
+       ///< If Id is 0 any existing id will be left untouched and only the
+       ///< given Language and Description will be set.
+       ///< \return Returns true if the track was set correctly, false otherwise.
+  const tTrackId *GetTrack(eTrackType Type);
+       ///< Returns a pointer to the given track id, or NULL if Type is not
+       ///< less than ttMaxTrackTypes.
+  int NumAudioTracks(void) const;
+       ///< Returns the number of audio tracks that are currently available.
+       ///< This is just for information, to quickly find out whether there
+       ///< is more than one audio track.
+  eTrackType GetCurrentAudioTrack(void) { return currentAudioTrack; }
+  bool SetCurrentAudioTrack(eTrackType Type);
+       ///< Sets the current audio track to the given Type.
+       ///< \return Returns true if Type is a valid audio track, false otherwise.
+  void EnsureAudioTrack(bool Force = false);
+       ///< Makes sure an audio track is selected that is actually available.
+       ///< If Force is true, the language and Dolby Digital settings will
+       ///< be verified even if the current audio track is available.
+
 // Audio facilities
 
 private:
   bool mute;
   int volume;
 protected:
+  virtual int GetAudioChannelDevice(void);
+       ///< Gets the current audio channel, which is stereo (0), mono left (1) or
+       ///< mono right (2).
+  virtual void SetAudioChannelDevice(int AudioChannel);
+       ///< Sets the audio channel to stereo (0), mono left (1) or mono right (2).
   virtual void SetVolumeDevice(int Volume);
        ///< Sets the audio volume on this device (Volume = 0...255).
-  virtual int NumAudioTracksDevice(void) const;
-       ///< Returns the number of audio tracks that are currently available on this
-       ///< device. The default return value is 0, meaning that this device
-       ///< doesn't have multiple audio track capabilities. The return value may
-       ///< change with every call and need not necessarily be the number of list
-       ///< entries returned by GetAudioTracksDevice(). This function is mainly called to
-       ///< decide whether there should be an "Audio" button in a menu.
-  virtual const char **GetAudioTracksDevice(int *CurrentTrack = NULL) const;
-       ///< Returns a list of currently available audio tracks. The last entry in the
-       ///< list must be NULL. The number of entries does not necessarily have to be
-       ///< the same as returned by a previous call to NumAudioTracksDevice().
-       ///< If CurrentTrack is given, it will be set to the index of the current track
-       ///< in the returned list. Note that the list must not be changed after it has
-       ///< been returned by a call to GetAudioTracksDevice()! The only time the list may
-       ///< change is *inside* the GetAudioTracksDevice() function.
-       ///< By default the return value is NULL and CurrentTrack, if given, will not
-       ///< have any meaning.
-  virtual void SetAudioTrackDevice(int Index);
-       ///< Sets the current audio track to the given value, which should be within the
-       ///< range of the list returned by a previous call to GetAudioTracksDevice()
-       ///< (otherwise nothing will happen).
+  virtual void SetDigitalAudioDevice(bool On);
+       ///< Tells the actual device that digital audio output shall be switched
+       ///< on or off.
 public:
   bool IsMute(void) const { return mute; }
   bool ToggleMute(void);
        ///< Turns the volume off or on and returns the new mute state.
+  int GetAudioChannel(void);
+       ///< Gets the current audio channel, which is stereo (0), mono left (1) or
+       ///< mono right (2).
+  void SetAudioChannel(int AudioChannel);
+       ///< Sets the audio channel to stereo (0), mono left (1) or mono right (2).
+       ///< Any other values will be silently ignored.
   void SetVolume(int Volume, bool Absolute = false);
        ///< Sets the volume to the given value, either absolutely or relative to
        ///< the current volume.
   static int CurrentVolume(void) { return primaryDevice ? primaryDevice->volume : 0; }//XXX???
-  int NumAudioTracks(void) const;
-       ///< Returns the number of audio tracks that are currently available on this
-       ///< device or a player attached to it.
-  const char **GetAudioTracks(int *CurrentTrack = NULL) const;
-       ///< Returns a list of currently available audio tracks. The last entry in the
-       ///< list is NULL. The number of entries does not necessarily have to be
-       ///< the same as returned by a previous call to NumAudioTracks().
-       ///< If CurrentTrack is given, it will be set to the index of the current track
-       ///< in the returned list.
-       ///< By default the return value is NULL and CurrentTrack, if given, will not
-       ///< have any meaning.
-  void SetAudioTrack(int Index);
-       ///< Sets the current audio track to the given value, which should be within the
-       ///< range of the list returned by a previous call to GetAudioTracks() (otherwise
-       ///< nothing will happen).
 
 // Player facilities
 
 private:
   cPlayer *player;
+  cPesAssembler *pesAssembler;
 protected:
   virtual bool CanReplay(void) const;
        ///< Returns true if this device can currently start a replay session.
   virtual bool SetPlayMode(ePlayMode PlayMode);
        ///< Sets the device into the given play mode.
        ///< \return true if the operation was successful.
+  virtual int PlayVideo(const uchar *Data, int Length);
+       ///< Plays the given data block as video.
+       ///< Data points to exactly one complete PES packet of the given Length.
+       ///< PlayVideo() shall process the packet either as a whole (returning
+       ///< Length) or not at all (returning 0 or -1 and setting 'errno' to EAGAIN).
+       ///< \return Returns the number of bytes actually taken from Data, or -1
+       ///< in case of an error.
+  virtual int PlayAudio(const uchar *Data, int Length);
+       ///< Plays the given data block as audio.
+       ///< Data points to exactly one complete PES packet of the given Length.
+       ///< PlayAudio() shall process the packet either as a whole (returning
+       ///< Length) or not at all (returning 0 or -1 and setting 'errno' to EAGAIN).
+       ///< \return Returns the number of bytes actually taken from Data, or -1
+       ///< in case of an error.
+  virtual int PlayPesPacket(const uchar *Data, int Length, bool VideoOnly = false);
+       ///< Plays the single PES packet in Data with the given Length.
+       ///< If VideoOnly is true, only the video will be displayed,
+       ///< which is necessary for trick modes like 'fast forward'.
+       ///< Data must point to one single, complete PES packet.
 public:
   virtual int64_t GetSTC(void);
        ///< Gets the current System Time Counter, which can be used to
@@ -349,14 +446,22 @@ public:
        ///< If TimeoutMs is not zero, the device will wait up to the given number
        ///< of milleseconds before returning in case there is no immediate
        ///< need for data.
-  virtual int PlayVideo(const uchar *Data, int Length);
-       ///< Actually plays the given data block as video. The data must be
-       ///< part of a PES (Packetized Elementary Stream) which can contain
-       ///< one video and one audio strem.
-  virtual void PlayAudio(const uchar *Data, int Length);
-       ///< Plays additional audio streams, like Dolby Digital.
-       ///< A derived class must call the base class function to make sure data
-       ///< is distributed to all registered cAudio objects.
+  virtual bool Flush(int TimeoutMs = 0);
+       ///< Returns true if the device's output buffers are empty, i. e. any
+       ///< data which was bufferd so far has been processed.
+       ///< If TimeoutMs is not zero, the device will wait up to the given
+       ///< number of milliseconds before returning in case there is still
+       ///< data in the buffers..
+  virtual int PlayPes(const uchar *Data, int Length, bool VideoOnly = false);
+       ///< Plays all valid PES packets in Data with the given Length.
+       ///< If Data is NULL any leftover data from a previous call will be
+       ///< discarded. If VideoOnly is true, only the video will be displayed,
+       ///< which is necessary for trick modes like 'fast forward'.
+       ///< Data should point to a sequence of complete PES packets. If the
+       ///< last packet in Data is not complete, it will be copied and combined
+       ///< to a complete packet with data from the next call to PlayPes().
+       ///< That way any functions called from within PlayPes() will be
+       ///< guaranteed to always receive complete PES packets.
   bool Replaying(void) const;
        ///< Returns true if we are currently replaying.
   void StopReplay(void);
@@ -369,6 +474,7 @@ public:
 // Receiver facilities
 
 private:
+  cMutex mutexReceiver;
   cReceiver *receiver[MAXRECEIVERS];
   int CanShift(int Ca, int Priority, int UsedCards = 0) const;
 protected:
@@ -397,6 +503,8 @@ public:
        ///< Attaches the given receiver to this device.
   void Detach(cReceiver *Receiver);
        ///< Detaches the given receiver from this device.
+  void DetachAll(int Pid);
+       ///< Detaches all receivers from this device for this pid.
   };
 
 /// Derived cDevice classes that can receive channels will have to provide
@@ -405,22 +513,19 @@ public:
 /// from the driver with each call to Read(), thus avoiding the overhead
 /// of getting each TS packet separately from the driver. It also makes
 /// sure the returned data points to a TS packet and automatically
-/// re-synchronizes after broken packet.
+/// re-synchronizes after broken packets.
 
-class cTSBuffer {
+class cTSBuffer : public cThread {
 private:
   int f;
-  int size;
   int cardIndex;
-  int tsRead;
-  int tsWrite;
-  uchar *buf;
-  bool firstRead;
-  int Used(void) { return tsRead <= tsWrite ? tsWrite - tsRead : size - tsRead + tsWrite; }
+  bool active;
+  bool delivered;
+  cRingBufferLinear *ringBuffer;
+  virtual void Action(void);
 public:
   cTSBuffer(int File, int Size, int CardIndex);
   ~cTSBuffer();
-  int Read(void);
   uchar *Get(void);
   };
 
