@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: dvbdevice.h 1.24 2003/11/07 13:17:13 kls Exp $
+ * $Id: dvbdevice.h 1.34 2005/02/20 11:17:07 kls Exp $
  */
 
 #ifndef __DVBDEVICE_H
@@ -14,7 +14,6 @@
 #include <linux/dvb/version.h>
 #include "device.h"
 #include "dvbspu.h"
-#include "eit.h"
 
 #if DVB_API_VERSION != 3
 #error VDR requires Linux DVB driver API version 3!
@@ -27,7 +26,6 @@ class cDvbTuner;
 /// The cDvbDevice implements a DVB device which can be accessed through the Linux DVB driver API.
 
 class cDvbDevice : public cDevice {
-  friend class cDvbOsd;
 private:
   static bool Probe(const char *FileName);
          ///< Probes for existing DVB devices.
@@ -39,38 +37,45 @@ public:
 private:
   fe_type_t frontendType;
   int fd_osd, fd_audio, fd_video, fd_dvr, fd_stc;
-  int OsdDeviceHandle(void) const { return fd_osd; }
 protected:
   virtual void MakePrimaryDevice(bool On);
 public:
   cDvbDevice(int n);
   virtual ~cDvbDevice();
+  virtual int ProvidesCa(const cChannel *Channel) const;
   virtual bool HasDecoder(void) const;
 
-// OSD facilities
+// SPU facilities
 
 private:
   cDvbSpuDecoder *spuDecoder;
 public:
-  cOsdBase *NewOsd(int x, int y);
   virtual cSpuDecoder *GetSpuDecoder(void);
 
 // Channel facilities
 
 private:
   cDvbTuner *dvbTuner;
-  void TurnOffLiveMode(void);
+  void TurnOffLiveMode(bool LiveView);
 public:
   virtual bool ProvidesSource(int Source) const;
+  virtual bool ProvidesTransponder(const cChannel *Channel) const;
   virtual bool ProvidesChannel(const cChannel *Channel, int Priority = -1, bool *NeedsDetachReceivers = NULL) const;
 protected:
   virtual bool SetChannelDevice(const cChannel *Channel, bool LiveView);
+public:
+  virtual bool HasLock(int TimeoutMs = 0);
 
 // PID handle facilities
 
 protected:
   virtual bool SetPid(cPidHandle *Handle, int Type, bool On);
 
+// Section filter facilities
+
+protected:
+  virtual int OpenFilter(u_short Pid, u_char Tid, u_char Mask);
+
 // Image Grab facilities
 
 private:
@@ -82,23 +87,27 @@ public:
 // Video format facilities
 
 public:
+  virtual void SetVideoDisplayFormat(eVideoDisplayFormat VideoDisplayFormat);
   virtual void SetVideoFormat(bool VideoFormat16_9);
   virtual eVideoSystem GetVideoSystem(void);
 
+// Track facilities
+
+protected:
+  virtual void SetAudioTrackDevice(eTrackType Type);
+
 // Audio facilities
 
 private:
-  int aPid1, aPid2;
+  bool digitalAudio;
+  static bool setTransferModeForDolbyDigital;
 protected:
+  virtual int GetAudioChannelDevice(void);
+  virtual void SetAudioChannelDevice(int AudioChannel);
   virtual void SetVolumeDevice(int Volume);
-  virtual int NumAudioTracksDevice(void) const;
-  virtual const char **GetAudioTracksDevice(int *CurrentTrack = NULL) const;
-  virtual void SetAudioTrackDevice(int Index);
-
-// EIT facilities
-
-private:
-  cSIProcessor *siProcessor;
+  virtual void SetDigitalAudioDevice(bool On);
+public:
+  static void SetTransferModeForDolbyDigital(bool On);
 
 // Player facilities
 
@@ -106,6 +115,8 @@ protected:
   ePlayMode playMode;
   virtual bool CanReplay(void) const;
   virtual bool SetPlayMode(ePlayMode PlayMode);
+  virtual int PlayVideo(const uchar *Data, int Length);
+  virtual int PlayAudio(const uchar *Data, int Length);
 public:
   virtual int64_t GetSTC(void);
   virtual void TrickSpeed(int Speed);
@@ -115,8 +126,7 @@ public:
   virtual void Mute(void);
   virtual void StillPicture(const uchar *Data, int Length);
   virtual bool Poll(cPoller &Poller, int TimeoutMs = 0);
-  virtual int PlayVideo(const uchar *Data, int Length);
-  virtual void PlayAudio(const uchar *Data, int Length);
+  virtual bool Flush(int TimeoutMs = 0);
 
 // Receiver facilities
 
