@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: dvbplayer.c 1.22 2003/05/24 09:04:26 kls Exp $
+ * $Id: dvbplayer.c 1.34 2005/05/22 11:26:51 kls Exp $
  */
 
 #include "dvbplayer.h"
@@ -80,8 +80,7 @@ private:
   int length;
   bool hasData;
   bool active;
-  cMutex mutex; 
-  cCondVar newSet;
+  cCondWait newSet;
 protected:
   void Action(void);
 public:
@@ -93,6 +92,7 @@ public:
   };
 
 cNonBlockingFileReader::cNonBlockingFileReader(void)
+:cThread("non blocking file reader")
 {
   f = -1;
   buffer = NULL;
@@ -105,20 +105,21 @@ cNonBlockingFileReader::cNonBlockingFileReader(void)
 cNonBlockingFileReader::~cNonBlockingFileReader()
 {
   active = false;
-  newSet.Broadcast();
+  newSet.Signal();
   Cancel(3);
   free(buffer);
 }
 
 void cNonBlockingFileReader::Clear(void)
 {
-  cMutexLock MutexLock(&mutex);
+  Lock();
   f = -1;
   free(buffer);
   buffer = NULL;
   wanted = length = 0;
   hasData = false;
-  newSet.Broadcast();
+  Unlock();
+  newSet.Signal();
 }
 
 int cNonBlockingFileReader::Read(int FileHandle, uchar *Buffer, int Length)
@@ -138,7 +139,7 @@ int cNonBlockingFileReader::Read(int FileHandle, uchar *Buffer, int Length)
      wanted = Length;
      length = 0;
      hasData = false;
-     newSet.Broadcast();
+     newSet.Signal();
      }
   errno = EAGAIN;
   return -1;
@@ -146,10 +147,9 @@ int cNonBlockingFileReader::Read(int FileHandle, uchar *Buffer, int Length)
 
 void cNonBlockingFileReader::Action(void)
 {
-  dsyslog("non blocking file reader thread started (pid=%d)", getpid());
   active = true;
   while (active) {
-        cMutexLock MutexLock(&mutex);
+        Lock();
         if (!hasData && f >= 0 && buffer) {
            int r = safe_read(f, buffer + length, wanted - length);
            if (r >= 0) {
@@ -163,17 +163,14 @@ void cNonBlockingFileReader::Action(void)
               hasData = true;
               }
            }
-        newSet.TimedWait(mutex, 1000);
+        Unlock();
+        newSet.Wait(1000);
         }
-  dsyslog("non blocking file reader thread ended (pid=%d)", getpid());
 }
 
 // --- cDvbPlayer ------------------------------------------------------------
 
-//XXX+ also used in recorder.c - find a better place???
-// The size of the array used to buffer video data:
-// (must be larger than MINVIDEODATA - see remux.h)
-#define VIDEOBUFSIZE  MEGABYTE(1)
+#define PLAYERBUFSIZE  MEGABYTE(1)
 
 // The number of frames to back up when resuming an interrupted replay session:
 #define RESUMEBACKUP (10 * FRAMESPERSEC)
@@ -197,13 +194,10 @@ private:
   ePlayDirs playDir;
   int trickSpeed;
   int readIndex, writeIndex;
-  bool canToggleAudioTrack;
-  uchar audioTrack;
   cFrame *readFrame;
   cFrame *playFrame;
   void TrickSpeed(int Increment);
   void Empty(void);
-  void StripAudioPackets(uchar *b, int Length, uchar Except = 0x00);
   bool NextFile(uchar FileNumber = 0, int FileOffset = -1);
   int Resume(void);
   bool Save(void);
@@ -223,9 +217,6 @@ public:
   void Goto(int Position, bool Still = false);
   virtual bool GetIndex(int &Current, int &Total, bool SnapToIFrame = false);
   virtual bool GetReplayMode(bool &Play, bool &Forward, int &Speed);
-  virtual int NumAudioTracks(void) const;
-  virtual const char **GetAudioTracks(int *CurrentTrack = NULL) const;
-  virtual void SetAudioTrack(int Index);
   };
 
 #define MAX_VIDEO_SLOWMOTION 63 // max. arg to pass to VIDEO_SLOWMOTION // TODO is this value correct?
@@ -235,6 +226,7 @@ public:
 int cDvbPlayer::Speeds[] = { 0, -2, -4, -8, 1, 2, 4, 12, 0 };
 
 cDvbPlayer::cDvbPlayer(const char *FileName)
+:cThread("dvbplayer")
 {
   nonBlockingFileReader = NULL;
   ringBuffer = NULL;
@@ -247,8 +239,6 @@ cDvbPlayer::cDvbPlayer(const char *FileName)
   playMode = pmPlay;
   playDir = pdForward;
   trickSpeed = NORMAL_SPEED;
-  canToggleAudioTrack = false;
-  audioTrack = 0xC0;
   readIndex = writeIndex = -1;
   readFrame = NULL;
   playFrame = NULL;
@@ -257,7 +247,7 @@ cDvbPlayer::cDvbPlayer(const char *FileName)
   replayFile = fileName->Open();
   if (replayFile < 0)
      return;
-  ringBuffer = new cRingBufferFrame(VIDEOBUFSIZE);
+  ringBuffer = new cRingBufferFrame(PLAYERBUFSIZE);
   // Create the index file:
   index = new cIndexFile(FileName, false);
   if (!index)
@@ -273,6 +263,7 @@ cDvbPlayer::~cDvbPlayer()
 {
   Detach();
   Save();
+  delete readFrame; // might not have been stored in the buffer in Action()
   delete index;
   delete fileName;
   delete backTrace;
@@ -306,6 +297,7 @@ void cDvbPlayer::Empty(void)
      nonBlockingFileReader->Clear();
   if ((readIndex = backTrace->Get(playDir == pdForward)) < 0)
      readIndex = writeIndex;
+  delete readFrame; // might not have been stored in the buffer in Action()
   readFrame = NULL;
   playFrame = NULL;
   ringBuffer->Clear();
@@ -314,41 +306,6 @@ void cDvbPlayer::Empty(void)
   firstPacket = true;
 }
 
-void cDvbPlayer::StripAudioPackets(uchar *b, int Length, uchar Except)
-{
-  if (index) {
-     for (int i = 0; i < Length - 6; i++) {
-         if (b[i] == 0x00 && b[i + 1] == 0x00 && b[i + 2] == 0x01) {
-            uchar c = b[i + 3];
-            int l = b[i + 4] * 256 + b[i + 5] + 6;
-            switch (c) {
-              case 0xBD: // dolby
-                   if (Except)
-                      PlayAudio(&b[i], l);
-                   // continue with deleting the data - otherwise it disturbs DVB replay
-              case 0xC0 ... 0xC1: // audio
-                   if (c == 0xC1)
-                      canToggleAudioTrack = true;
-                   if (!Except || c != Except)
-                      memset(&b[i], 0x00, min(l, Length-i));
-                   break;
-              case 0xE0 ... 0xEF: // video
-                   break;
-              default:
-                   //esyslog("ERROR: unexpected packet id %02X", c);
-                   l = 0;
-              }
-            if (l)
-               i += l - 1; // the loop increments, too!
-            }
-         /*XXX
-         else
-            esyslog("ERROR: broken packet header");
-            XXX*/
-         }
-     }
-}
-
 bool cDvbPlayer::NextFile(uchar FileNumber, int FileOffset)
 {
   if (FileNumber > 0)
@@ -398,29 +355,31 @@ void cDvbPlayer::Activate(bool On)
      }
   else if (active) {
      running = false;
-     Cancel(3);
+     Cancel(9);
      active = false;
      }
 }
 
 void cDvbPlayer::Action(void)
 {
-  dsyslog("dvbplayer thread started (pid=%d)", getpid());
-
   uchar *b = NULL;
   uchar *p = NULL;
   int pc = 0;
 
   readIndex = Resume();
   if (readIndex >= 0)
-     isyslog("resuming replay at index %d (%s)", readIndex, IndexToHMSF(readIndex, true));
+     isyslog("resuming replay at index %d (%s)", readIndex, *IndexToHMSF(readIndex, true));
 
   nonBlockingFileReader = new cNonBlockingFileReader;
   int Length = 0;
-  int AudioTrack = 0; // -1 = any, 0 = none, >0 = audioTrack
+  bool Sleep = false;
 
   running = true;
-  while (running && (NextFile() || readIndex >= 0 || ringBuffer->Available())) {
+  while (running && (NextFile() || readIndex >= 0 || ringBuffer->Available() || !DeviceFlush(100))) {
+        if (Sleep) {
+           cCondWait::SleepMs(3); // this keeps the CPU load low
+           Sleep = false;
+           }
         cPoller Poller;
         if (DevicePoll(Poller, 100)) {
 
@@ -428,8 +387,8 @@ void cDvbPlayer::Action(void)
 
            // Read the next frame from the file:
 
-           if (!readFrame && (replayFile >= 0 || readIndex >= 0)) {
-              if (playMode != pmStill) {
+           if (playMode != pmStill && playMode != pmPause) {
+              if (!readFrame && (replayFile >= 0 || readIndex >= 0)) {
                  if (!nonBlockingFileReader->Reading()) {
                     if (playMode == pmFast || (playMode == pmSlow && playDir == pdBackward)) {
                        uchar FileNumber;
@@ -440,6 +399,10 @@ void cDvbPlayer::Action(void)
                              continue;
                           }
                        else {
+                          // hit begin of recording: wait for device buffers to drain
+                          // before changing play mode:
+                          if (!DeviceFlush(100))
+                             continue;
                           // can't call Play() here, because those functions may only be
                           // called from the foreground thread - and we also don't need
                           // to empty the buffer here
@@ -449,9 +412,6 @@ void cDvbPlayer::Action(void)
                           continue;
                           }
                        readIndex = Index;
-                       AudioTrack = 0;
-                       // must clear all audio packets because the buffer is not emptied
-                       // when falling back from "fast forward" to "play" (see above)
                        }
                     else if (index) {
                        uchar FileNumber;
@@ -462,12 +422,9 @@ void cDvbPlayer::Action(void)
                           eof = true;
                           continue;
                           }
-                       AudioTrack = audioTrack;
                        }
-                    else { // allows replay even if the index file is missing
+                    else // allows replay even if the index file is missing
                        Length = MAXFRAMESIZE;
-                       AudioTrack = -1;
-                       }
                     if (Length == -1)
                        Length = MAXFRAMESIZE; // this means we read up to EOF (see cIndex)
                     else if (Length > MAXFRAMESIZE) {
@@ -478,8 +435,6 @@ void cDvbPlayer::Action(void)
                     }
                  int r = nonBlockingFileReader->Read(replayFile, b, Length);
                  if (r > 0) {
-                    if (AudioTrack == 0)
-                       StripAudioPackets(b, r);
                     readFrame = new cFrame(b, -r, ftUnknown, readIndex); // hands over b to the ringBuffer
                     b = NULL;
                     }
@@ -490,16 +445,16 @@ void cDvbPlayer::Action(void)
                     break;
                     }
                  }
-              else//XXX
-                 usleep(1); // this keeps the CPU load low
-              }
 
-           // Store the frame in the buffer:
+              // Store the frame in the buffer:
 
-           if (readFrame) {
-              if (ringBuffer->Put(readFrame))
-                 readFrame = NULL;
+              if (readFrame) {
+                 if (ringBuffer->Put(readFrame))
+                    readFrame = NULL;
+                 }
               }
+           else
+              Sleep = true;
 
            // Get the next frame from the buffer:
 
@@ -517,15 +472,14 @@ void cDvbPlayer::Action(void)
                  pc = playFrame->Count();
                  if (p) {
                     if (firstPacket) {
+                       PlayPes(NULL, 0);
                        cRemux::SetBrokenLink(p, pc);
                        firstPacket = false;
                        }
-                    if (AudioTrack > 0)
-                       StripAudioPackets(p, pc, AudioTrack);
                     }
                  }
               if (p) {
-                 int w = PlayVideo(p, pc);
+                 int w = PlayPes(p, pc, playMode != pmPlay);
                  if (w > 0) {
                     p += w;
                     pc -= w;
@@ -550,8 +504,6 @@ void cDvbPlayer::Action(void)
   cNonBlockingFileReader *nbfr = nonBlockingFileReader;
   nonBlockingFileReader = NULL;
   delete nbfr;
-
-  dsyslog("dvbplayer thread ended (pid=%d)", getpid());
 }
 
 void cDvbPlayer::Pause(void)
@@ -719,7 +671,6 @@ void cDvbPlayer::Goto(int Index, bool Still)
         if (r > 0) {
            if (playMode == pmPause)
               DevicePlay();
-           StripAudioPackets(b, r);
            DeviceStillPicture(b, r);
            }
         playMode = pmStill;
@@ -759,31 +710,6 @@ bool cDvbPlayer::GetReplayMode(bool &Play, bool &Forward, int &Speed)
   return true;
 }
 
-int cDvbPlayer::NumAudioTracks(void) const
-{
-  return canToggleAudioTrack ? 2 : 1;
-}
-
-const char **cDvbPlayer::GetAudioTracks(int *CurrentTrack) const
-{
-  if (NumAudioTracks()) {
-     if (CurrentTrack)
-        *CurrentTrack = (audioTrack == 0xC0) ? 0 : 1;
-     static const char *audioTracks1[] = { "Audio 1", NULL };
-     static const char *audioTracks2[] = { "Audio 1", "Audio 2", NULL };
-     return NumAudioTracks() > 1 ? audioTracks2 : audioTracks1;
-     }
-  return NULL;
-}
-
-void cDvbPlayer::SetAudioTrack(int Index)
-{
-  if ((audioTrack == 0xC0) != (Index == 0)) {
-     audioTrack = (Index == 1) ? 0xC1 : 0xC0;
-     Empty();
-     }
-}
-
 // --- cDvbPlayerControl -----------------------------------------------------
 
 cDvbPlayerControl::cDvbPlayerControl(const char *FileName)
