@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: menu.h 1.57 2003/08/03 09:37:18 kls Exp $
+ * $Id: menu.h 1.70 2005/05/15 14:34:54 kls Exp $
  */
 
 #ifndef __MENU_H
@@ -12,10 +12,22 @@
 
 #include "ci.h"
 #include "device.h"
-#include "osd.h"
+#include "epg.h"
+#include "osdbase.h"
 #include "dvbplayer.h"
 #include "recorder.h"
-#include "recording.h"
+#include "skins.h"
+
+class cMenuText : public cOsdMenu {
+private:
+  char *text;
+public:
+  cMenuText(const char *Title, const char *Text, eDvbFont Font = fontOsd);
+  virtual ~cMenuText();
+  void SetText(const char *Text);
+  virtual void Display(void);
+  virtual eOSState ProcessKey(eKeys Key);
+  };
 
 class cMenuMain : public cOsdMenu {
 private:
@@ -31,12 +43,15 @@ public:
 
 class cDisplayChannel : public cOsdObject {
 private:
+  cSkinDisplayChannel *displayChannel;
   int group;
   bool withInfo;
-  int lines;
-  int lastTime;
+  cTimeMs lastTime;
   int number;
-  void DisplayChannel(const cChannel *Channel);
+  cChannel *channel;
+  const cEvent *lastPresent;
+  const cEvent *lastFollowing;
+  void DisplayChannel(void);
   void DisplayInfo(void);
   void Refresh(void);
 public:
@@ -48,8 +63,9 @@ public:
 
 class cDisplayVolume : public cOsdObject {
 private:
-  int timeout;
-  static cDisplayVolume *displayVolume;
+  cSkinDisplayVolume *displayVolume;
+  cTimeMs timeout;
+  static cDisplayVolume *currentDisplayVolume;
   virtual void Show(void);
   cDisplayVolume(void);
 public:
@@ -59,6 +75,24 @@ public:
   eOSState ProcessKey(eKeys Key);
   };
 
+class cDisplayTracks : public cOsdObject {
+private:
+  cSkinDisplayTracks *displayTracks;
+  cTimeMs timeout;
+  eTrackType types[ttMaxTrackTypes];
+  char *descriptions[ttMaxTrackTypes];
+  int numTracks, track, audioChannel;
+  static cDisplayTracks *currentDisplayTracks;
+  virtual void Show(void);
+  cDisplayTracks(void);
+public:
+  virtual ~cDisplayTracks();
+  static bool IsOpen(void) { return currentDisplayTracks != NULL; }
+  static cDisplayTracks *Create(void);
+  static void Process(eKeys Key);
+  eOSState ProcessKey(eKeys Key);
+  };
+
 class cMenuCam : public cOsdMenu {
 private:
   cCiMenu *ciMenu;
@@ -90,7 +124,6 @@ class cMenuRecordingItem;
 
 class cMenuRecordings : public cOsdMenu {
 private:
-  static cRecordings Recordings;
   char *base;
   int level;
   static int helpKeys;
@@ -100,7 +133,7 @@ private:
   eOSState Play(void);
   eOSState Rewind(void);
   eOSState Delete(void);
-  eOSState Summary(void);
+  eOSState Info(void);
   eOSState Commands(eKeys Key = kNone);
 public:
   cMenuRecordings(const char *Base = NULL, int Level = 0, bool OpenSubMenus = false);
@@ -113,17 +146,16 @@ private:
   cDevice *device;
   cTimer *timer;
   cRecorder *recorder;
-  const cEventInfo *eventInfo;
+  const cEvent *event;
   char *instantId;
   char *fileName;
-  bool GetEventInfo(void);
+  bool GetEvent(void);
 public:
   cRecordControl(cDevice *Device, cTimer *Timer = NULL, bool Pause = false);
   virtual ~cRecordControl();
   bool Process(time_t t);
-  bool Uses(cDevice *Device) { return Device == device; }
-  void Stop(bool KeepInstant = false);
-  bool IsInstant(void) { return instantId; }
+  cDevice *Device(void) { return device; }
+  void Stop(void);
   const char *InstantId(void) { return instantId; }
   const char *FileName(void) { return fileName; }
   cTimer *Timer(void) { return timer; }
@@ -141,17 +173,21 @@ public:
   static const char *GetInstantId(const char *LastInstantId);
   static cRecordControl *GetRecordControl(const char *FileName);
   static void Process(time_t t);
+  static void ChannelDataModified(cChannel *Channel);
   static bool Active(void);
   static void Shutdown(void);
   };
 
 class cReplayControl : public cDvbPlayerControl {
 private:
+  cSkinDisplayReplay *displayReplay;
   cMarks marks;
   bool visible, modeOnly, shown, displayFrames;
   int lastCurrent, lastTotal;
+  bool lastPlay, lastForward;
+  int lastSpeed;
   time_t timeoutShow;
-  bool timeSearchActive, timeSearchHide;  
+  bool timeSearchActive, timeSearchHide;
   int timeSearchTime, timeSearchPos;
   void TimeSearchDisplay(void);
   void TimeSearchProcess(eKeys Key);
@@ -159,7 +195,6 @@ private:
   void ShowTimed(int Seconds = 0);
   static char *fileName;
   static char *title;
-  void DisplayAtBottom(const char *s = NULL);
   void ShowMode(void);
   bool ShowProgress(bool Initial);
   void MarkToggle(void);
