@@ -4,13 +4,15 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: menuitems.c 1.13 2003/04/12 09:21:33 kls Exp $
+ * $Id: menuitems.c 1.22 2005/03/19 15:33:34 kls Exp $
  */
 
 #include "menuitems.h"
 #include <ctype.h>
 #include "i18n.h"
 #include "plugin.h"
+#include "remote.h"
+#include "skins.h"
 #include "status.h"
 
 const char *FileNameChars = " abcdefghijklmnopqrstuvwxyz0123456789-.#~";
@@ -36,7 +38,7 @@ void cMenuEditItem::SetValue(const char *Value)
   char *buffer = NULL;
   asprintf(&buffer, "%s:\t%s", name, value);
   SetText(buffer, false);
-  Display();
+  cStatus::MsgOsdCurrentItem(buffer);
 }
 
 // --- cMenuEditIntItem ------------------------------------------------------
@@ -112,6 +114,23 @@ void cMenuEditBoolItem::Set(void)
   SetValue(buf);
 }
 
+// --- cMenuEditBitItem ------------------------------------------------------
+
+cMenuEditBitItem::cMenuEditBitItem(const char *Name, int *Value, int Mask, const char *FalseString, const char *TrueString)
+:cMenuEditBoolItem(Name, &bit, FalseString, TrueString)
+{
+  value = Value;
+  bit = (*value & Mask) != 0;
+  mask = Mask;
+  Set();
+}
+
+void cMenuEditBitItem::Set(void)
+{
+  *value = bit ? *value | mask : *value & ~mask;
+  cMenuEditBoolItem::Set();
+}
+
 // --- cMenuEditNumItem ------------------------------------------------------
 
 cMenuEditNumItem::cMenuEditNumItem(const char *Name, char *Value, int Length, bool Blind)
@@ -235,9 +254,9 @@ cMenuEditStrItem::~cMenuEditStrItem()
 void cMenuEditStrItem::SetHelpKeys(void)
 {
   if (pos >= 0)
-     Interface->Help(tr("ABC/abc"), tr(insert ? "Overwrite" : "Insert"), tr("Delete"));
+     cSkinDisplay::Current()->SetButtons(tr("ABC/abc"), tr(insert ? "Overwrite" : "Insert"), tr("Delete"));
   else
-     Interface->Help(NULL);
+     cSkinDisplay::Current()->SetButtons(NULL);
 }
 
 void cMenuEditStrItem::Set(void)
@@ -246,21 +265,21 @@ void cMenuEditStrItem::Set(void)
   const char *fmt = insert && newchar ? "[]%c%s" : "[%c]%s";
 
   if (pos >= 0) {
+     const cFont *font = cFont::GetFont(fontOsd);
      strncpy(buf, value, pos);
      snprintf(buf + pos, sizeof(buf) - pos - 2, fmt, *(value + pos), value + pos + 1);
-     int width = Interface->Width() - Interface->GetCols()[0];
-     if (cOsd::WidthInCells(buf) <= width) {
+     int width = cSkinDisplay::Current()->EditableWidth();
+     if (font->Width(buf) <= width) {
         // the whole buffer fits on the screen
         SetValue(buf);
         return;
         }
-     width *= cOsd::CellWidth();
-     width -= cOsd::Width('>'); // assuming '<' and '>' have the same with
+     width -= font->Width('>'); // assuming '<' and '>' have the same with
      int w = 0;
      int i = 0;
      int l = strlen(buf);
      while (i < l && w <= width)
-           w += cOsd::Width(buf[i++]);
+           w += font->Width(buf[i++]);
      if (i >= pos + 4) {
         // the cursor fits on the screen
         buf[i - 1] = '>';
@@ -277,7 +296,7 @@ void cMenuEditStrItem::Set(void)
      else
         i--;
      while (i >= 0 && w <= width)
-           w += cOsd::Width(buf[i--]);
+           w += font->Width(buf[i--]);
      buf[++i] = '<';
      SetValue(buf + i);
      }
@@ -312,8 +331,8 @@ eOSState cMenuEditStrItem::ProcessKey(eKeys Key)
                  if (pos >= 0) {
                     insert = !insert;
                     newchar = true;
+                    SetHelpKeys();
                     }
-                 SetHelpKeys();
                  break;
     case kYellow|k_Repeat:
     case kYellow: // Remove the character at current position; in insert mode it is the character to the right of cursor
@@ -443,100 +462,295 @@ void cMenuEditStraItem::Set(void)
   SetValue(strings[*value]);
 }
 
-// --- cMenuTextItem ---------------------------------------------------------
+// --- cMenuEditChanItem -----------------------------------------------------
 
-cMenuTextItem::cMenuTextItem(const char *Text, int X, int Y, int W, int H, eDvbColor FgColor, eDvbColor BgColor, eDvbFont Font)
+cMenuEditChanItem::cMenuEditChanItem(const char *Name, int *Value)
+:cMenuEditIntItem(Name, Value, 1, Channels.MaxNumber())
 {
-  x = X;
-  y = Y;
-  w = W;
-  h = H;
-  fgColor = FgColor;
-  bgColor = BgColor;
-  font = Font;
-  offset = 0;
-  eDvbFont oldFont = Interface->SetFont(font);
-  text = Interface->WrapText(Text, w - 1, &lines);
-  Interface->SetFont(oldFont);
-  if (h < 0)
-     h = lines;
+  Set();
 }
 
-cMenuTextItem::~cMenuTextItem()
+void cMenuEditChanItem::Set(void)
 {
-  free(text);
+  char buf[255];
+  cChannel *channel = Channels.GetByNumber(*value);
+  snprintf(buf, sizeof(buf), "%d %s", *value, channel ? channel->Name() : "");
+  SetValue(buf);
 }
 
-void cMenuTextItem::Clear(void)
+eOSState cMenuEditChanItem::ProcessKey(eKeys Key)
 {
-  Interface->Fill(x, y, w, h, bgColor);
-}
+  int delta = 1;
 
-void cMenuTextItem::Display(int Offset, eDvbColor FgColor, eDvbColor BgColor)
-{
-  int l = 0;
-  char *t = text;
-  eDvbFont oldFont = Interface->SetFont(font);
-  while (*t) {
-        char *n = strchr(t, '\n');
-        if (l >= offset) {
-           if (n)
-              *n = 0;
-           Interface->Write(x, y + l - offset, t, fgColor, bgColor);
-           if (n)
-              *n = '\n';
-           else
-              break;
-           }
-        if (!n)
-           break;
-        t = n + 1;
-        if (++l >= h + offset)
-           break;
-        }
-  Interface->SetFont(oldFont);
-  // scroll indicators use inverted color scheme!
-  if (CanScrollUp())   Interface->Write(x + w - 1, y,         "^", bgColor, fgColor);
-  if (CanScrollDown()) Interface->Write(x + w - 1, y + h - 1, "v", bgColor, fgColor);
-  cStatus::MsgOsdTextItem(text);
-}
-
-void cMenuTextItem::ScrollUp(bool Page)
-{
-  if (CanScrollUp()) {
-     Clear();
-     offset = max(offset - (Page ? h : 1), 0);
-     Display();
-     }
-  cStatus::MsgOsdTextItem(NULL, true);
-}
-
-void cMenuTextItem::ScrollDown(bool Page)
-{
-  if (CanScrollDown()) {
-     Clear();
-     offset = min(offset + (Page ? h : 1), lines - h);
-     Display();
-     }
-  cStatus::MsgOsdTextItem(NULL, false);
-}
-
-eOSState cMenuTextItem::ProcessKey(eKeys Key)
-{
   switch (Key) {
     case kLeft|k_Repeat:
-    case kLeft:
-    case kUp|k_Repeat:
-    case kUp:            ScrollUp(NORMALKEY(Key) == kLeft);    break;
+    case kLeft:  delta = -1;
     case kRight|k_Repeat:
     case kRight:
-    case kDown|k_Repeat:
-    case kDown:          ScrollDown(NORMALKEY(Key) == kRight); break;
-    default:             return osUnknown;
+                 {
+                   cChannel *channel = Channels.GetByNumber(*value + delta, delta);
+                   if (channel) {
+                      *value = channel->Number();
+                      Set();
+                      }
+                 }
+                 break;
+    default: return cMenuEditIntItem::ProcessKey(Key);
     }
   return osContinue;
 }
 
+// --- cMenuEditTranItem -----------------------------------------------------
+
+cMenuEditTranItem::cMenuEditTranItem(const char *Name, int *Value, int *Source)
+:cMenuEditChanItem(Name, Value)
+{
+  number = 0;
+  source = Source;
+  transponder = *Value;
+  cChannel *channel = Channels.First();
+  while (channel) {
+        if (!channel->GroupSep() && *source == channel->Source() && ISTRANSPONDER(channel->Transponder(), *Value)) {
+           number = channel->Number();
+           break;
+           }
+        channel = (cChannel *)channel->Next();
+        }
+  *Value = number;
+  Set();
+  *Value = transponder;
+}
+
+eOSState cMenuEditTranItem::ProcessKey(eKeys Key)
+{
+  *value = number;
+  eOSState state = cMenuEditChanItem::ProcessKey(Key);
+  number = *value;
+  cChannel *channel = Channels.GetByNumber(*value);
+  if (channel) {
+     *source = channel->Source();
+     transponder = channel->Transponder();
+     }
+  *value = transponder;
+  return state;
+}
+
+// --- cMenuEditDateItem -----------------------------------------------------
+
+static int ParseWeekDays(const char *s)
+{
+  time_t day;
+  int weekdays;
+  return cTimer::ParseDay(s, day, weekdays) ? weekdays : 0;
+}
+
+int cMenuEditDateItem::days[] = { ParseWeekDays("M------"),
+                                  ParseWeekDays("-T-----"),
+                                  ParseWeekDays("--W----"),
+                                  ParseWeekDays("---T---"),
+                                  ParseWeekDays("----F--"),
+                                  ParseWeekDays("-----S-"),
+                                  ParseWeekDays("------S"),
+                                  ParseWeekDays("MTWTF--"),
+                                  ParseWeekDays("MTWTFS-"),
+                                  ParseWeekDays("MTWTFSS"),
+                                  ParseWeekDays("-----SS"),
+                                  0 };
+
+cMenuEditDateItem::cMenuEditDateItem(const char *Name, time_t *Value, int *WeekDays)
+:cMenuEditItem(Name)
+{
+  value = Value;
+  weekdays = WeekDays;
+  oldvalue = 0;
+  dayindex = 0;
+  Set();
+}
+
+void cMenuEditDateItem::Set(void)
+{
+#define DATEBUFFERSIZE 32
+  char buf[DATEBUFFERSIZE];
+  if (weekdays && *weekdays) {
+     SetValue(cTimer::PrintDay(0, *weekdays));
+     return;
+     }
+  else if (*value) {
+     struct tm tm_r;
+     localtime_r(value, &tm_r);
+     strftime(buf, DATEBUFFERSIZE, "%Y-%m-%d ", &tm_r);
+     strcat(buf, WeekDayName(tm_r.tm_wday));
+     }
+  else
+     *buf = 0;
+  SetValue(buf);
+}
+
+eOSState cMenuEditDateItem::ProcessKey(eKeys Key)
+{
+  eOSState state = cMenuEditItem::ProcessKey(Key);
+
+  if (state == osUnknown) {
+     time_t now = time(NULL);
+     if (NORMALKEY(Key) == kLeft) { // TODO might want to increase the delta if repeated quickly?
+        if (!weekdays || !*weekdays) {
+           // Decrement single day:
+           time_t v = *value;
+           v -= SECSINDAY;
+           if (v < now) {
+              if (now <= v + SECSINDAY) { // switched from tomorrow to today
+                 if (!weekdays)
+                    v = 0;
+                 }
+              else if (weekdays) { // switched from today to yesterday, so enter weekdays mode
+                 v = 0;
+                 dayindex = sizeof(days) / sizeof(int) - 2;
+                 *weekdays = days[dayindex];
+                 }
+              else // don't go before today
+                 v = *value;
+              }
+           *value = v;
+           }
+        else {
+           // Decrement weekday index:
+           if (dayindex > 0)
+              *weekdays = days[--dayindex];
+           }
+        }
+     else if (NORMALKEY(Key) == kRight) {
+        if (!weekdays || !*weekdays) {
+           // Increment single day:
+           if (!*value)
+              *value = cTimer::SetTime(now, 0);
+           *value += SECSINDAY;
+           }
+        else {
+           // Increment weekday index:
+           *weekdays = days[++dayindex];
+           if (!*weekdays) { // was last weekday entry, so switch to today
+              *value = cTimer::SetTime(now, 0);
+              dayindex = 0;
+              }
+           }
+        }
+     else if (weekdays) {
+        if (Key == k0) {
+           // Toggle between weekdays and single day:
+           if (*weekdays) {
+              *value = cTimer::SetTime(oldvalue ? oldvalue : now, 0);
+              oldvalue = 0;
+              *weekdays = 0;
+              }
+           else {
+              *weekdays = days[cTimer::GetWDay(*value)];
+              oldvalue = *value;
+              *value = 0;
+              }
+           }
+        else if (k1 <= Key && Key <= k7) {
+           // Toggle individual weekdays:
+           if (*weekdays) {
+              int v = *weekdays ^ (1 << (Key - k1));
+              if (v != 0)
+                 *weekdays = v; // can't let this become all 0
+              }
+           }
+        else
+           return state;
+        }
+     else
+        return state;
+     Set();
+     state = osContinue;
+     }
+  return state;
+}
+
+// --- cMenuEditTimeItem -----------------------------------------------------
+
+cMenuEditTimeItem::cMenuEditTimeItem(const char *Name, int *Value)
+:cMenuEditItem(Name)
+{
+  value = Value;
+  hh = *value / 100;
+  mm = *value % 100;
+  pos = 0;
+  Set();
+}
+
+void cMenuEditTimeItem::Set(void)
+{
+  char buf[10];
+  switch (pos) {
+    case 1:  snprintf(buf, sizeof(buf), "%01d-:--", hh / 10); break;
+    case 2:  snprintf(buf, sizeof(buf), "%02d:--", hh); break;
+    case 3:  snprintf(buf, sizeof(buf), "%02d:%01d-", hh, mm / 10); break;
+    default: snprintf(buf, sizeof(buf), "%02d:%02d", hh, mm);
+    }
+  SetValue(buf);
+}
+
+eOSState cMenuEditTimeItem::ProcessKey(eKeys Key)
+{
+  eOSState state = cMenuEditItem::ProcessKey(Key);
+
+  if (state == osUnknown) {
+     if (k0 <= Key && Key <= k9) {
+        if (fresh || pos > 3) {
+           pos = 0;
+           fresh = false;
+           }
+        int n = Key - k0;
+        switch (pos) {
+          case 0: if (n <= 2) {
+                     hh = n * 10;
+                     mm = 0;
+                     pos++;
+                     }
+                  break;
+          case 1: if (hh + n <= 23) {
+                     hh += n;
+                     pos++;
+                     }
+                  break;
+          case 2: if (n <= 5) {
+                     mm += n * 10;
+                     pos++;
+                     }
+                  break;
+          case 3: if (mm + n <= 59) {
+                     mm += n;
+                     pos++;
+                     }
+                  break;
+          }
+        }
+     else if (NORMALKEY(Key) == kLeft) { // TODO might want to increase the delta if repeated quickly?
+        if (--mm < 0) {
+           mm = 59;
+           if (--hh < 0)
+              hh = 23;
+           }
+        fresh = true;
+        }
+     else if (NORMALKEY(Key) == kRight) {
+        if (++mm > 59) {
+           mm = 0;
+           if (++hh > 23)
+              hh = 0;
+           }
+        fresh = true;
+        }
+     else
+        return state;
+     *value = hh * 100 + mm;
+     Set();
+     state = osContinue;
+     }
+  return state;
+}
+
 // --- cMenuSetupPage --------------------------------------------------------
 
 cMenuSetupPage::cMenuSetupPage(void)
