@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: plugin.c 1.10 2003/08/30 14:52:58 kls Exp $
+ * $Id: plugin.c 1.13 2005/01/30 14:05:20 kls Exp $
  */
 
 #include "plugin.h"
@@ -28,6 +28,7 @@ char *cPlugin::configDirectory = NULL;
 cPlugin::cPlugin(void) 
 {
   name = NULL;
+  started = false;
 }
 
 cPlugin::~cPlugin()
@@ -60,6 +61,10 @@ bool cPlugin::Start(void)
   return true;
 }
 
+void cPlugin::Stop(void)
+{
+}
+
 void cPlugin::Housekeeping(void)
 {
 }
@@ -162,7 +167,7 @@ bool cDll::Load(bool Log)
   const char *error = dlerror();
   if (!error) {
      void *(*creator)(void);
-     (void *)creator = dlsym(handle, "VDRPluginCreator");
+     creator = (void *(*)(void))dlsym(handle, "VDRPluginCreator");
      if (!(error = dlerror()))
         plugin = (cPlugin *)creator();
      }
@@ -256,26 +261,23 @@ void cPluginManager::SetDirectory(const char *Directory)
 void cPluginManager::AddPlugin(const char *Args)
 {
   if (strcmp(Args, "*") == 0) {
-     DIR *d = opendir(directory);
-     if (d) {
-        struct dirent *e;
-        while ((e = readdir(d)) != NULL) {
-              if (strstr(e->d_name, LIBVDR_PREFIX) == e->d_name) {
-                 char *p = strstr(e->d_name, SO_INDICATOR);
-                 if (p) {
-                    *p = 0;
-                    p += strlen(SO_INDICATOR);
-                    if (strcmp(p, VDRVERSION) == 0) {
-                       char *name = e->d_name + strlen(LIBVDR_PREFIX);
-                       if (strcmp(name, "*") != 0) { // let's not get into a loop!
-                          AddPlugin(e->d_name + strlen(LIBVDR_PREFIX));
-                          }
+     cReadDir d(directory);
+     struct dirent *e;
+     while ((e = d.Next()) != NULL) {
+           if (strstr(e->d_name, LIBVDR_PREFIX) == e->d_name) {
+              char *p = strstr(e->d_name, SO_INDICATOR);
+              if (p) {
+                 *p = 0;
+                 p += strlen(SO_INDICATOR);
+                 if (strcmp(p, VDRVERSION) == 0) {
+                    char *name = e->d_name + strlen(LIBVDR_PREFIX);
+                    if (strcmp(name, "*") != 0) { // let's not get into a loop!
+                       AddPlugin(e->d_name + strlen(LIBVDR_PREFIX));
                        }
                     }
                  }
               }
-        closedir(d);
-        }
+           }
      return;
      }
   char *s = strdup(skipspace(Args));
@@ -325,6 +327,7 @@ bool cPluginManager::StartPlugins(void)
          Setup.OSDLanguage = Language;
          if (!p->Start())
             return false;
+         p->started = true;
          }
       }
   return true;
@@ -369,15 +372,25 @@ cPlugin *cPluginManager::GetPlugin(const char *Name)
   return NULL;
 }
 
-void cPluginManager::Shutdown(bool Log)
+void cPluginManager::StopPlugins(void)
+{
+  for (cDll *dll = dlls.Last(); dll; dll = dlls.Prev(dll)) {
+      cPlugin *p = dll->Plugin();
+      if (p && p->started) {
+         isyslog("stopping plugin: %s", p->Name());
+         p->Stop();
+         p->started = false;
+         }
+      }
+}
+
+void cPluginManager::Shutdown(void)
 {
   cDll *dll;
   while ((dll = dlls.Last()) != NULL) {
-        if (Log) {
-           cPlugin *p = dll->Plugin();
-           if (p)
-              isyslog("stopping plugin: %s", p->Name());
-           }
+        cPlugin *p = dll->Plugin();
+        if (p)
+           isyslog("deleting plugin: %s", p->Name());
         dlls.Del(dll);
         }
 }
