@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: recorder.h 1.1 2002/06/08 09:35:03 kls Exp $
+ * $Id: recorder.h 1.3 2005/01/15 16:35:53 kls Exp $
  */
 
 #ifndef __RECORDER_H
@@ -16,25 +16,20 @@
 #include "ringbuffer.h"
 #include "thread.h"
 
+class cFileWriter;
+
 class cRecorder : public cReceiver, cThread {
 private:
   cRingBufferLinear *ringBuffer;
   cRemux *remux;
-  cFileName *fileName;
-  cIndexFile *index;
-  uchar pictureType;
-  int fileSize;
-  int recordFile;
+  cFileWriter *writer;
   bool active;
-  time_t lastDiskSpaceCheck;
-  bool RunningLowOnDiskSpace(void);
-  bool NextFile(void);
 protected:
   virtual void Activate(bool On);
   virtual void Receive(uchar *Data, int Length);
   virtual void Action(void);
 public:
-  cRecorder(const char *FileName, int Ca, int Priority, int VPid, int APid1, int APid2, int DPid1, int DPid2);
+  cRecorder(const char *FileName, int Ca, int Priority, int VPid, const int *APids, const int *DPids, const int *SPids);
                // Creates a new recorder that requires conditional access Ca, has
                // the given Priority and will record the given PIDs into the file FileName.
   virtual ~cRecorder();
