@@ -4,10 +4,11 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: recording.c 1.86 2003/11/08 15:25:35 kls Exp $
+ * $Id: recording.c 1.109 2005/06/05 14:11:45 kls Exp $
  */
 
 #include "recording.h"
+#include <dirent.h>
 #include <errno.h>
 #include <fcntl.h>
 #include <stdio.h>
@@ -18,13 +19,16 @@
 #include "i18n.h"
 #include "interface.h"
 #include "remux.h" //XXX+ I_FRAME
+#include "skins.h"
 #include "tools.h"
 #include "videodir.h"
 
+#define SUMMARYFALLBACK
+
 #define RECEXT       ".rec"
 #define DELEXT       ".del"
 /* This was the original code, which works fine in a Linux only environment.
-   Unfortunately, because of windows and its brain dead file system, we have
+   Unfortunately, because of Windows and its brain dead file system, we have
    to use a more complicated approach, in order to allow users who have enabled
    the VFAT compile time option to see their recordings even if they forget to
    enable VFAT when compiling a new version of VDR... Gee, do I hate Windows.
@@ -43,11 +47,12 @@
 // end of implementation for brain dead systems
 
 #define RESUMEFILESUFFIX  "/resume%s%s.vdr"
+#ifdef SUMMARYFALLBACK
 #define SUMMARYFILESUFFIX "/summary.vdr"
+#endif
+#define INFOFILESUFFIX    "/info.vdr"
 #define MARKSFILESUFFIX   "/marks.vdr"
 
-#define FINDCMD      "cd '%s' && find '%s' -follow -type d -name '%s' 2> /dev/null"
-
 #define MINDISKSPACE 1024 // MB
 
 #define DELETEDLIFETIME     1 // hours after which a deleted recording will be actually removed
@@ -69,14 +74,14 @@ void RemoveDeletedRecordings(void)
      if (!LockFile.Lock())
         return;
      // Remove the oldest file that has been "deleted":
-     cRecordings Recordings;
-     if (Recordings.Load(true)) {
-        cRecording *r = Recordings.First();
+     cRecordings DeletedRecordings(true);
+     if (DeletedRecordings.Load()) {
+        cRecording *r = DeletedRecordings.First();
         cRecording *r0 = r;
         while (r) {
               if (r->start < r0->start)
                  r0 = r;
-              r = Recordings.Next(r);
+              r = DeletedRecordings.Next(r);
               }
         if (r0 && time(NULL) - r0->start > DELETEDLIFETIME * 3600) {
            r0->Remove();
@@ -104,14 +109,14 @@ void AssertFreeDiskSpace(int Priority)
            return;
         // Remove the oldest file that has been "deleted":
         isyslog("low disk space while recording, trying to remove a deleted recording...");
-        cRecordings Recordings;
-        if (Recordings.Load(true)) {
-           cRecording *r = Recordings.First();
+        cRecordings DeletedRecordings(true);
+        if (DeletedRecordings.Load()) {
+           cRecording *r = DeletedRecordings.First();
            cRecording *r0 = r;
            while (r) {
                  if (r->start < r0->start)
                     r0 = r;
-                 r = Recordings.Next(r);
+                 r = DeletedRecordings.Next(r);
                  }
            if (r0 && r0->Remove()) {
               LastFreeDiskCheck += REMOVELATENCY / Factor;
@@ -120,13 +125,13 @@ void AssertFreeDiskSpace(int Priority)
            }
         // No "deleted" files to remove, so let's see if we can delete a recording:
         isyslog("...no deleted recording found, trying to delete an old recording...");
-        if (Recordings.Load(false)) {
+        if (Recordings.Load()) {
            cRecording *r = Recordings.First();
            cRecording *r0 = NULL;
            while (r) {
                  if (!r->IsEdited() && r->lifetime < MAXLIFETIME) { // edited recordings and recordings with MAXLIFETIME live forever
                     if ((r->lifetime == 0 && Priority > r->priority) || // the recording has no guaranteed lifetime and the new recording has higher priority
-                        (time(NULL) - r->start) / SECSINDAY > r->lifetime) { // the recording's guaranteed lifetime has expired
+                        (r->lifetime > 0 && (time(NULL) - r->start) / SECSINDAY >= r->lifetime)) { // the recording's guaranteed lifetime has expired
                        if (r0) {
                           if (r->priority < r0->priority || (r->priority == r0->priority && r->start < r0->start))
                              r0 = r; // in any case we delete the one with the lowest priority (or the older one in case of equal priorities)
@@ -137,8 +142,10 @@ void AssertFreeDiskSpace(int Priority)
                     }
                  r = Recordings.Next(r);
                  }
-           if (r0 && r0->Delete())
+           if (r0 && r0->Delete()) {
+              Recordings.Del(r0);
               return;
+              }
            }
         // Unable to free disk space, but there's nothing we can do about that...
         isyslog("...no old recording found, giving up");
@@ -155,7 +162,7 @@ cResumeFile::cResumeFile(const char *FileName)
   fileName = MALLOC(char, strlen(FileName) + strlen(RESUMEFILESUFFIX) + 1);
   if (fileName) {
      strcpy(fileName, FileName);
-     sprintf(fileName + strlen(fileName), RESUMEFILESUFFIX, Setup.ResumeID ? "." : "", Setup.ResumeID ? itoa(Setup.ResumeID) : "");
+     sprintf(fileName + strlen(fileName), RESUMEFILESUFFIX, Setup.ResumeID ? "." : "", Setup.ResumeID ? *itoa(Setup.ResumeID) : "");
      }
   else
      esyslog("ERROR: can't allocate memory for resume file name");
@@ -211,6 +218,68 @@ void cResumeFile::Delete(void)
      }
 }
 
+// --- cRecordingInfo --------------------------------------------------------
+
+cRecordingInfo::cRecordingInfo(const cEvent *Event)
+{
+  if (Event) {
+     event = Event;
+     channelID = event->ChannelID();
+     ownEvent = NULL;
+     }
+  else
+     event = ownEvent = new cEvent(0);
+}
+
+cRecordingInfo::~cRecordingInfo()
+{
+  delete ownEvent;
+}
+
+void cRecordingInfo::SetData(const char *Title, const char *ShortText, const char *Description)
+{
+  if (!isempty(Title))
+     ((cEvent *)event)->SetTitle(Title);
+  if (!isempty(ShortText))
+     ((cEvent *)event)->SetShortText(ShortText);
+  if (!isempty(Description))
+     ((cEvent *)event)->SetDescription(Description);
+}
+
+bool cRecordingInfo::Read(FILE *f)
+{
+  if (ownEvent) {
+     cReadLine ReadLine;
+     char *s;
+     while ((s = ReadLine.Read(f)) != NULL) {
+           char *t = skipspace(s + 1);
+           switch (*s) {
+             case 'C': {
+                         char *p = strchr(t, ' ');
+                         if (p)
+                            *p = 0; // strips optional channel name
+                         if (*t)
+                            channelID = tChannelID::FromString(t);
+                       }
+                       break;
+             default: if (!ownEvent->Parse(s))
+                         return false;
+                      break;
+             }
+           }
+     return true;
+     }
+  return false;
+}
+
+bool cRecordingInfo::Write(FILE *f, const char *Prefix) const
+{
+  if (channelID.Valid())
+     fprintf(f, "%sC %s\n", Prefix, *channelID.ToString());
+  event->Dump(f, Prefix, true);
+  return true;
+}
+
 // --- cRecording ------------------------------------------------------------
 
 #define RESUME_NOT_INITIALIZED (-2)
@@ -306,7 +375,7 @@ static char *ExchangeChars(char *s, bool ToFileSystem)
   return s;
 }
 
-cRecording::cRecording(cTimer *Timer, const char *Title, const char *Subtitle, const char *Summary)
+cRecording::cRecording(cTimer *Timer, const cEvent *Event)
 {
   resume = RESUME_NOT_INITIALIZED;
   titleBuffer = NULL;
@@ -314,7 +383,8 @@ cRecording::cRecording(cTimer *Timer, const char *Title, const char *Subtitle, c
   fileName = NULL;
   name = NULL;
   // set up the actual name:
-  const char *OriginalSubtitle = Subtitle;
+  const char *Title = Event ? Event->Title() : NULL;
+  const char *Subtitle = Event ? Event->ShortText() : NULL;
   char SubtitleBuffer[MAX_SUBTITLE_LENGTH];
   if (isempty(Title))
      Title = Timer->Channel()->Name();
@@ -331,9 +401,17 @@ cRecording::cRecording(cTimer *Timer, const char *Title, const char *Subtitle, c
      name = strdup(Timer->File());
      name = strreplace(name, TIMERMACRO_TITLE, Title);
      name = strreplace(name, TIMERMACRO_EPISODE, Subtitle);
+     // avoid blanks at the end:
+     int l = strlen(name);
+     while (l-- > 2) {
+           if (name[l] == ' ' && name[l - 1] != '~')
+              name[l] = 0;
+           else
+              break;
+           }
      if (Timer->IsSingleEvent()) {
         Timer->SetFile(name); // this was an instant recording, so let's set the actual data
-        Timers.Save();
+        Timers.SetModified();
         }
      }
   else if (Timer->IsSingleEvent() || !Setup.UseSubtitle)
@@ -345,17 +423,13 @@ cRecording::cRecording(cTimer *Timer, const char *Title, const char *Subtitle, c
   start = Timer->StartTime();
   priority = Timer->Priority();
   lifetime = Timer->Lifetime();
-  // handle summary:
-  summary = !isempty(Timer->Summary()) ? strdup(Timer->Summary()) : NULL;
-  if (!summary) {
-     Subtitle = OriginalSubtitle;
-     if (isempty(Subtitle))
-        Subtitle = "";
-     if (isempty(Summary))
-        Summary = "";
-     if (*Subtitle || *Summary)
-        asprintf(&summary, "%s\n\n%s%s%s", Title, Subtitle, (*Subtitle && *Summary) ? "\n\n" : "", Summary);
-     }
+  // handle info:
+  info = new cRecordingInfo(Event);
+  // this is a somewhat ugly hack to get the 'summary' information from the
+  // timer into the recording info, but it saves us from having to actually
+  // copy the entire event data:
+  if (!isempty(Timer->Summary()))
+     info->SetData(isempty(info->Title()) ? Timer->File() : NULL, NULL, Timer->Summary());
 }
 
 cRecording::cRecording(const char *FileName)
@@ -368,7 +442,7 @@ cRecording::cRecording(const char *FileName)
   char *p = strrchr(FileName, '/');
 
   name = NULL;
-  summary = NULL;
+  info = new cRecordingInfo;
   if (p) {
      time_t now = time(NULL);
      struct tm tm_r;
@@ -384,39 +458,57 @@ cRecording::cRecording(const char *FileName)
         name[p - FileName] = 0;
         name = ExchangeChars(name, false);
         }
-     // read an optional summary file:
-     char *SummaryFileName = NULL;
-     asprintf(&SummaryFileName, "%s%s", fileName, SUMMARYFILESUFFIX);
-     int f = open(SummaryFileName, O_RDONLY);
-     if (f >= 0) {
-        struct stat buf;
-        if (fstat(f, &buf) == 0) {
-           int size = buf.st_size;
-           summary = MALLOC(char, size + 1); // +1 for terminating 0
-           if (summary) {
-              int rbytes = safe_read(f, summary, size);
-              if (rbytes >= 0) {
-                 summary[rbytes] = 0;
-                 if (rbytes != size)
-                    esyslog("%s: expected %d bytes but read %d", SummaryFileName, size, rbytes);
-                 }
-              else {
-                 LOG_ERROR_STR(SummaryFileName);
-                 free(summary);
-                 summary = NULL;
-                 }
-
-              }
-           else
-              esyslog("can't allocate %d byte of memory for summary file '%s'", size + 1, SummaryFileName);
-           close(f);
-           }
-        else
-           LOG_ERROR_STR(SummaryFileName);
+     // read an optional info file:
+     char *InfoFileName = NULL;
+     asprintf(&InfoFileName, "%s%s", fileName, INFOFILESUFFIX);
+     FILE *f = fopen(InfoFileName, "r");
+     if (f) {
+        info->Read(f);
+        fclose(f);
         }
      else if (errno != ENOENT)
-        LOG_ERROR_STR(SummaryFileName);
-     free(SummaryFileName);
+        LOG_ERROR_STR(InfoFileName);
+     free(InfoFileName);
+#ifdef SUMMARYFALLBACK
+     // fall back to the old 'summary.vdr' if there was no 'info.vdr':
+     if (isempty(info->Title())) {
+        char *SummaryFileName = NULL;
+        asprintf(&SummaryFileName, "%s%s", fileName, SUMMARYFILESUFFIX);
+        FILE *f = fopen(SummaryFileName, "r");
+        if (f) {
+           int line = 0;
+           char *data[3] = { NULL };
+           cReadLine ReadLine;
+           char *s;
+           while ((s = ReadLine.Read(f)) != NULL) {
+                 if (*s || line > 1) {
+                    if (data[line]) {
+                       int len = strlen(s);
+                       len += strlen(data[line]) + 1;
+                       data[line] = (char *)realloc(data[line], len + 1);
+                       strcat(data[line], "\n");
+                       strcat(data[line], s);
+                       }
+                    else
+                       data[line] = strdup(s);
+                    }
+                 else
+                    line++;
+                 }
+           fclose(f);
+           if (line == 1) {
+              data[2] = data[1];
+              data[1] = NULL;
+              }
+           info->SetData(data[0], data[1], data[2]);
+           for (int i = 0; i < 3; i ++)
+               free(data[i]);
+           }
+        else if (errno != ENOENT)
+           LOG_ERROR_STR(SummaryFileName);
+        free(SummaryFileName);
+        }
+#endif
      }
 }
 
@@ -426,7 +518,7 @@ cRecording::~cRecording()
   free(sortBuffer);
   free(fileName);
   free(name);
-  free(summary);
+  delete info;
 }
 
 char *cRecording::StripEpisodeName(char *s)
@@ -449,11 +541,11 @@ char *cRecording::StripEpisodeName(char *s)
   return s;
 }
 
-char *cRecording::SortName(void)
+char *cRecording::SortName(void) const
 {
   if (!sortBuffer) {
      char *s = StripEpisodeName(strdup(FileName() + strlen(VideoDirectory) + 1));
-     int l = strxfrm(NULL, s, 0);
+     int l = strxfrm(NULL, s, 0) + 1;
      sortBuffer = MALLOC(char, l);
      strxfrm(sortBuffer, s, l);
      free(s);
@@ -461,7 +553,7 @@ char *cRecording::SortName(void)
   return sortBuffer;
 }
 
-int cRecording::GetResume(void)
+int cRecording::GetResume(void) const
 {
   if (resume == RESUME_NOT_INITIALIZED) {
      cResumeFile ResumeFile(FileName());
@@ -470,13 +562,13 @@ int cRecording::GetResume(void)
   return resume;
 }
 
-bool cRecording::operator< (const cListObject &ListObject)
+int cRecording::Compare(const cListObject &ListObject) const
 {
   cRecording *r = (cRecording *)&ListObject;
-  return strcasecmp(SortName(), r->SortName()) < 0;
+  return strcasecmp(SortName(), r->SortName());
 }
 
-const char *cRecording::FileName(void)
+const char *cRecording::FileName(void) const
 {
   if (!fileName) {
      struct tm tm_r;
@@ -488,7 +580,7 @@ const char *cRecording::FileName(void)
   return fileName;
 }
 
-const char *cRecording::Title(char Delimiter, bool NewIndicator, int Level)
+const char *cRecording::Title(char Delimiter, bool NewIndicator, int Level) const
 {
   char New = NewIndicator && IsNew() ? '*' : ' ';
   free(titleBuffer);
@@ -501,9 +593,10 @@ const char *cRecording::Title(char Delimiter, bool NewIndicator, int Level)
         s++;
      else
         s = name;
-     asprintf(&titleBuffer, "%02d.%02d%c%02d:%02d%c%c%s",
+     asprintf(&titleBuffer, "%02d.%02d.%02d%c%02d:%02d%c%c%s",
                             t->tm_mday,
                             t->tm_mon + 1,
+                            t->tm_year % 100,
                             Delimiter,
                             t->tm_hour,
                             t->tm_min,
@@ -511,7 +604,8 @@ const char *cRecording::Title(char Delimiter, bool NewIndicator, int Level)
                             Delimiter,
                             s);
      // let's not display a trailing '~':
-     stripspace(titleBuffer);
+     if (!NewIndicator)
+        stripspace(titleBuffer);
      s = &titleBuffer[strlen(titleBuffer) - 1];
      if (*s == '~')
         *s = 0;
@@ -539,8 +633,8 @@ const char *cRecording::Title(char Delimiter, bool NewIndicator, int Level)
 
 const char *cRecording::PrefixFileName(char Prefix)
 {
-  const char *p = PrefixVideoFileName(FileName(), Prefix);
-  if (p) {
+  cString p = PrefixVideoFileName(FileName(), Prefix);
+  if (*p) {
      free(fileName);
      fileName = strdup(p);
      return fileName;
@@ -548,7 +642,7 @@ const char *cRecording::PrefixFileName(char Prefix)
   return NULL;
 }
 
-int cRecording::HierarchyLevels(void)
+int cRecording::HierarchyLevels(void) const
 {
   const char *s = name;
   int level = 0;
@@ -559,28 +653,25 @@ int cRecording::HierarchyLevels(void)
   return level;
 }
 
-bool cRecording::IsEdited(void)
+bool cRecording::IsEdited(void) const
 {
   const char *s = strrchr(name, '~');
   s = !s ? name : s + 1;
   return *s == '%';
 }
 
-bool cRecording::WriteSummary(void)
+bool cRecording::WriteInfo(void)
 {
-  if (summary) {
-     char *SummaryFileName = NULL;
-     asprintf(&SummaryFileName, "%s%s", fileName, SUMMARYFILESUFFIX);
-     FILE *f = fopen(SummaryFileName, "w");
-     if (f) {
-        if (fputs(summary, f) < 0)
-           LOG_ERROR_STR(SummaryFileName);
-        fclose(f);
-        }
-     else
-        LOG_ERROR_STR(SummaryFileName);
-     free(SummaryFileName);
+  char *InfoFileName = NULL;
+  asprintf(&InfoFileName, "%s%s", fileName, INFOFILESUFFIX);
+  FILE *f = fopen(InfoFileName, "w");
+  if (f) {
+     info->Write(f);
+     fclose(f);
      }
+  else
+     LOG_ERROR_STR(InfoFileName);
+  free(InfoFileName);
   return true;
 }
 
@@ -616,30 +707,64 @@ bool cRecording::Remove(void)
 
 // --- cRecordings -----------------------------------------------------------
 
-bool cRecordings::Load(bool Deleted)
+cRecordings Recordings;
+
+cRecordings::cRecordings(bool Deleted)
 {
-  Clear();
-  bool result = false;
-  char *cmd = NULL;
-  asprintf(&cmd, FINDCMD, VideoDirectory, VideoDirectory, Deleted ? "*" DELEXT : "*" RECEXT);
-  FILE *p = popen(cmd, "r");
-  if (p) {
-     char *s;
-     while ((s = readline(p)) != NULL) {
-           cRecording *r = new cRecording(s);
-           if (r->Name())
-              Add(r);
-           else
-              delete r;
+  deleted = Deleted;
+  lastUpdate = 0;
+}
+
+void cRecordings::ScanVideoDir(const char *DirName)
+{
+  cReadDir d(DirName);
+  struct dirent *e;
+  while ((e = d.Next()) != NULL) {
+        if (strcmp(e->d_name, ".") && strcmp(e->d_name, "..")) {
+           char *buffer;
+           asprintf(&buffer, "%s/%s", DirName, e->d_name);
+           struct stat st;
+           if (stat(buffer, &st) == 0) {
+              if (S_ISLNK(st.st_mode)) {
+                 char *old = buffer;
+                 buffer = ReadLink(old);
+                 free(old);
+                 if (!buffer)
+                    continue;
+                 if (stat(buffer, &st) != 0) {
+                    free(buffer);
+                    continue;
+                    }
+                 }
+              if (S_ISDIR(st.st_mode)) {
+                 if (endswith(buffer, deleted ? DELEXT : RECEXT)) {
+                    cRecording *r = new cRecording(buffer);
+                    if (r->Name())
+                       Add(r);
+                    else
+                       delete r;
+                    }
+                 else
+                    ScanVideoDir(buffer);
+                 }
+              }
+           free(buffer);
            }
-     pclose(p);
-     Sort();
-     result = Count() > 0;
-     }
-  else
-     Interface->Error("Error while opening pipe!");
-  free(cmd);
-  return result;
+        }
+}
+
+bool cRecordings::NeedsUpdate(void)
+{
+  return lastUpdate <= LastModifiedTime(AddDirectory(VideoDirectory, ".update"));
+}
+
+bool cRecordings::Load(void)
+{
+  lastUpdate = time(NULL); // doing this first to make sure we don't miss anything
+  Clear();
+  ScanVideoDir(VideoDirectory);
+  Sort();
+  return Count() > 0;
 }
 
 cRecording *cRecordings::GetByName(const char *FileName)
@@ -651,9 +776,23 @@ cRecording *cRecordings::GetByName(const char *FileName)
   return NULL;
 }
 
-// --- cMark -----------------------------------------------------------------
+void cRecordings::AddByName(const char *FileName)
+{
+  cRecording *recording = GetByName(FileName);
+  if (!recording) {
+     recording = new cRecording(FileName);
+     Add(recording);
+     }
+}
 
-char *cMark::buffer = NULL;
+void cRecordings::DelByName(const char *FileName)
+{
+  cRecording *recording = GetByName(FileName);
+  if (recording)
+     Del(recording);
+}
+
+// --- cMark -----------------------------------------------------------------
 
 cMark::cMark(int Position, const char *Comment)
 {
@@ -666,11 +805,11 @@ cMark::~cMark()
   free(comment);
 }
 
-const char *cMark::ToText(void)
+cString cMark::ToText(void)
 {
-  free(buffer);
-  asprintf(&buffer, "%s%s%s\n", IndexToHMSF(position, true), comment ? " " : "", comment ? comment : "");
-  return buffer;
+  char *buffer;
+  asprintf(&buffer, "%s%s%s\n", *IndexToHMSF(position, true), comment ? " " : "", comment ? comment : "");
+  return cString(buffer, true);
 }
 
 bool cMark::Parse(const char *s)
@@ -696,8 +835,7 @@ bool cMark::Save(FILE *f)
 
 bool cMarks::Load(const char *RecordingFileName)
 {
-  const char *MarksFile = AddDirectory(RecordingFileName, MARKSFILESUFFIX);
-  if (cConfig<cMark>::Load(MarksFile)) {
+  if (cConfig<cMark>::Load(AddDirectory(RecordingFileName, MARKSFILESUFFIX))) {
      Sort();
      return true;
      }
@@ -761,7 +899,7 @@ void cRecordingUserCommand::InvokeCommand(const char *State, const char *Recordi
 {
   if (command) {
      char *cmd;
-     asprintf(&cmd, "%s %s \"%s\"", command, State, strescape(RecordingFileName, "\"$"));
+     asprintf(&cmd, "%s %s \"%s\"", command, State, *strescape(RecordingFileName, "\"$"));
      isyslog("executing '%s'", cmd);
      SystemExec(cmd);
      free(cmd);
@@ -907,7 +1045,7 @@ bool cIndexFile::CatchUp(int Index)
             LOG_ERROR_STR(fileName);
          if (Index < last - (i ? 2 * INDEXSAFETYLIMIT : 0) || Index > 10 * INDEXSAFETYLIMIT) // keep off the end in case of "Pause live video"
             break;
-         sleep(1);
+         cCondWait::SleepMs(1000);
          }
      }
   return index != NULL;
@@ -1075,8 +1213,21 @@ int cFileName::SetOffset(int Number, int Offset)
      fileNumber = Number;
      sprintf(pFileNumber, RECORDFILESUFFIX, fileNumber);
      if (record) {
-        if (access(fileName, F_OK) == 0) // file exists, let's try next suffix
-           return SetOffset(Number + 1);
+        if (access(fileName, F_OK) == 0) {
+           // files exists, check if it has non-zero size
+           struct stat buf;
+           if (stat(fileName, &buf) == 0) {
+              if (buf.st_size != 0)
+                 return SetOffset(Number + 1); // file exists and has non zero size, let's try next suffix
+              else {
+                 // zero size file, remove it
+                 dsyslog ("cFileName::SetOffset: removing zero-sized file %s\n", fileName);
+                 unlink (fileName);
+                 }
+              }
+           else
+              return SetOffset(Number + 1); // error with fstat - should not happen, just to be on the safe side
+           }
         else if (errno != ENOENT) { // something serious has happened
            LOG_ERROR_STR(fileName);
            return -1;
@@ -1102,9 +1253,9 @@ int cFileName::NextFile(void)
 
 // --- Index stuff -----------------------------------------------------------
 
-const char *IndexToHMSF(int Index, bool WithFrame)
+cString IndexToHMSF(int Index, bool WithFrame)
 {
-  static char buffer[16];
+  char buffer[16];
   int f = (Index % FRAMESPERSEC) + 1;
   int s = (Index / FRAMESPERSEC);
   int m = s / 60 % 60;
