@@ -4,14 +4,16 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: recording.h 1.28 2003/10/17 14:27:36 kls Exp $
+ * $Id: recording.h 1.38 2005/05/28 09:34:07 kls Exp $
  */
 
 #ifndef __RECORDING_H
 #define __RECORDING_H
 
 #include <time.h>
+#include "channels.h"
 #include "config.h"
+#include "epg.h"
 #include "thread.h"
 #include "timers.h"
 #include "tools.h"
@@ -32,34 +34,52 @@ public:
   void Delete(void);
   };
 
+class cRecordingInfo {
+  friend class cRecording;
+private:
+  tChannelID channelID;
+  const cEvent *event;
+  cEvent *ownEvent;
+  cRecordingInfo(const cEvent *Event = NULL);
+  void SetData(const char *Title, const char *ShortText, const char *Description);
+public:
+  ~cRecordingInfo();
+  const char *Title(void) const { return event->Title(); }
+  const char *ShortText(void) const { return event->ShortText(); }
+  const char *Description(void) const { return event->Description(); }
+  const cComponents *Components(void) const { return event->Components(); }
+  bool Read(FILE *f);
+  bool Write(FILE *f, const char *Prefix = "") const;
+  };
+
 class cRecording : public cListObject {
 private:
-  int resume;
-  char *titleBuffer;
-  char *sortBuffer;
-  char *fileName;
-  char *name;
-  char *summary;
-  char *StripEpisodeName(char *s);
-  char *SortName(void);
-  int GetResume(void);
+  mutable int resume;
+  mutable char *titleBuffer;
+  mutable char *sortBuffer;
+  mutable char *fileName;
+  mutable char *name;
+  cRecordingInfo *info;
+  static char *StripEpisodeName(char *s);
+  char *SortName(void) const;
+  int GetResume(void) const;
 public:
   time_t start;
   int priority;
   int lifetime;
-  cRecording(cTimer *Timer, const char *Title, const char *Subtitle, const char *Summary);
+  cRecording(cTimer *Timer, const cEvent *Event);
   cRecording(const char *FileName);
   ~cRecording();
-  virtual bool operator< (const cListObject &ListObject);
-  const char *Name(void) { return name; }
-  const char *FileName(void);
-  const char *Title(char Delimiter = ' ', bool NewIndicator = false, int Level = -1);
-  const char *Summary(void) { return summary; }
+  virtual int Compare(const cListObject &ListObject) const;
+  const char *Name(void) const { return name; }
+  const char *FileName(void) const;
+  const char *Title(char Delimiter = ' ', bool NewIndicator = false, int Level = -1) const;
+  const cRecordingInfo *Info(void) const { return info; }
   const char *PrefixFileName(char Prefix);
-  int HierarchyLevels(void);
-  bool IsNew(void) { return GetResume() <= 0; }
-  bool IsEdited(void);
-  bool WriteSummary(void);
+  int HierarchyLevels(void) const;
+  bool IsNew(void) const { return GetResume() <= 0; }
+  bool IsEdited(void) const;
+  bool WriteInfo(void);
   bool Delete(void);
        // Changes the file name so that it will no longer be visible in the "Recordings" menu
        // Returns false in case of error
@@ -69,20 +89,29 @@ public:
   };
 
 class cRecordings : public cList<cRecording> {
+private:
+  bool deleted;
+  time_t lastUpdate;
+  void ScanVideoDir(const char *DirName);
 public:
-  bool Load(bool Deleted = false);
+  cRecordings(bool Deleted = false);
+  bool Load(void);
+  void TriggerUpdate(void) { lastUpdate = 0; }
+  bool NeedsUpdate(void);
   cRecording *GetByName(const char *FileName);
+  void AddByName(const char *FileName);
+  void DelByName(const char *FileName);
   };
 
+extern cRecordings Recordings;
+
 class cMark : public cListObject {
-private:
-  static char *buffer;
 public:
   int position;
   char *comment;
   cMark(int Position = 0, const char *Comment = NULL);
   ~cMark();
-  const char *ToText(void);
+  cString ToText(void);
   bool Parse(const char *s);
   bool Save(FILE *f);
   };
@@ -112,8 +141,8 @@ public:
 //XXX+
 #define FRAMESPERSEC 25
 
-// The maximum size of a single frame:
-#define MAXFRAMESIZE  KILOBYTE(192)
+// The maximum size of a single frame (up to HDTV 1920x1080):
+#define MAXFRAMESIZE  KILOBYTE(512)
 
 // The maximum file size is limited by the range that can be covered
 // with 'int'. 4GB might be possible (if the range is considered
@@ -165,7 +194,7 @@ public:
   int NextFile(void);
   };
 
-const char *IndexToHMSF(int Index, bool WithFrame = false);
+cString IndexToHMSF(int Index, bool WithFrame = false);
       // Converts the given index to a string, optionally containing the frame number.
 int HMSFToIndex(const char *HMSF);
       // Converts the given string (format: "hh:mm:ss.ff") to an index.
