@@ -6,7 +6,7 @@
  * This code is distributed under the terms and conditions of the
  * GNU GENERAL PUBLIC LICENSE. See the file COPYING for details.
  *
- * $Id: spu.h 1.1 2002/09/08 14:17:51 kls Exp $
+ * $Id: spu.h 1.4 2005/02/20 11:21:31 kls Exp $
  */
 
 #ifndef __SPU_VDR_H
@@ -21,10 +21,11 @@ class cSpuDecoder {
     typedef enum { eSpuNormal, eSpuLetterBox, eSpuPanAndScan } eScaleMode;
   public:
     //    cSpuDecoder();
-     virtual ~ cSpuDecoder();
+    virtual ~cSpuDecoder();
 
     virtual int setTime(uint32_t pts) = 0;
 
+    virtual cSpuDecoder::eScaleMode getScaleMode(void) = 0;
     virtual void setScaleMode(cSpuDecoder::eScaleMode ScaleMode) = 0;
     virtual void setPalette(uint32_t * pal) = 0;
     virtual void setHighlight(uint16_t sx, uint16_t sy,
@@ -32,7 +33,10 @@ class cSpuDecoder {
                               uint32_t palette) = 0;
     virtual void clearHighlight(void) = 0;
     virtual void Empty(void) = 0;
-    virtual void processSPU(uint32_t pts, uint8_t * buf) = 0;
+    virtual void Hide(void) = 0;
+    virtual void Draw(void) = 0;
+    virtual bool IsVisible(void) = 0;
+    virtual void processSPU(uint32_t pts, uint8_t * buf, bool AllowedShow = true) = 0;
 };
 
 #endif                          // __SPU_VDR_H
