@@ -10,7 +10,7 @@
  * and interact with the Video Disk Recorder - or write a full featured
  * graphical interface that sits on top of an SVDRP connection.
  *
- * $Id: svdrp.c 1.55 2003/08/31 11:24:47 kls Exp $
+ * $Id: svdrp.c 1.72 2005/05/26 09:59:09 kls Exp $
  */
 
 #include "svdrp.h"
@@ -29,7 +29,9 @@
 #include "channels.h"
 #include "config.h"
 #include "device.h"
+#include "eitscan.h"
 #include "keys.h"
+#include "menu.h"
 #include "remote.h"
 #include "timers.h"
 #include "tools.h"
@@ -110,7 +112,8 @@ int cSocket::Accept(void)
         bool accepted = SVDRPhosts.Acceptable(clientname.sin_addr.s_addr);
         if (!accepted) {
            const char *s = "Access denied!\n";
-           write(newsock, s, strlen(s));
+           if (write(newsock, s, strlen(s)) < 0)
+              LOG_ERROR;
            close(newsock);
            newsock = -1;
            }
@@ -155,7 +158,7 @@ bool cPUTEhandler::Process(const char *s)
      else {
         rewind(f);
         if (cSchedules::Read(f)) {
-           cSIProcessor::TriggerDump();
+           cSchedules::Cleanup(true);
            status = 250;
            message = "EPG data processed";
            }
@@ -204,11 +207,15 @@ const char *HelpPages[] = {
   "    List channels. Without option, all channels are listed. Otherwise\n"
   "    only the given channel is listed. If a name is given, all channels\n"
   "    containing the given string as part of their name are listed.",
-  "LSTE\n"
-  "    List EPG data.",
+  "LSTE [ <channel> ] [ now | next | at <time> ]\n"
+  "    List EPG data. Without any parameters all data of all channels is\n"
+  "    listed. If a channel is given (either by number of by channel ID),\n"
+  "    only data for that channel is listed. 'now', 'next', or 'at <time>'\n"
+  "    restricts the returned data to present events, following events, or\n"
+  "    events at the given time (which must be in time_t form).",
   "LSTR [ <number> ]\n"
   "    List recordings. Without option, all recordings are listed. Otherwise\n"
-  "    the summary for the given recording is listed.",
+  "    the information for the given recording is listed.",
   "LSTT [ <number> ]\n"
   "    List timers. Without option, all timers are listed. Otherwise\n"
   "    only the given timer is listed.",
@@ -251,6 +258,9 @@ const char *HelpPages[] = {
   "    format defined in vdr(5) for the 'epg.data' file.  A '.' on a line\n"
   "    by itself terminates the input and starts processing of the data (all\n"
   "    entered data is buffered until the terminating '.' is seen).",
+  "SCAN\n"
+  "    Forces an EPG scan. If this is a single DVB device system, the scan\n"
+  "    will be done on the primary device unless it is currently recording.",
   "STAT disk\n"
   "    Return information about disk usage (total, free, percent).",
   "UPDT <settings>\n"
@@ -272,7 +282,7 @@ const char *HelpPages[] = {
 /* SVDRP Reply Codes:
 
  214 Help message
- 215 EPG data record
+ 215 EPG or recording data record
  220 VDR service ready
  221 VDR service closing transmission channel
  250 Requested VDR action okay, completed
@@ -458,7 +468,7 @@ void cSVDRP::CmdCHAN(const char *Option)
 
 void cSVDRP::CmdCLRE(const char *Option)
 {
-  cSIProcessor::Clear();
+  cSchedules::ClearAll();
   Reply(250, "EPG data cleared");
 }
 
@@ -476,7 +486,7 @@ void cSVDRP::CmdDELC(const char *Option)
                }
            Channels.Del(channel);
            Channels.ReNumber();
-           Channels.Save();
+           Channels.SetModified(true);
            isyslog("channel %s deleted", Option);
            Reply(250, "Channel \"%s\" deleted", Option);
            }
@@ -496,10 +506,17 @@ void cSVDRP::CmdDELR(const char *Option)
      if (isnumber(Option)) {
         cRecording *recording = Recordings.Get(strtol(Option, NULL, 10) - 1);
         if (recording) {
-           if (recording->Delete())
-              Reply(250, "Recording \"%s\" deleted", Option);
+           cRecordControl *rc = cRecordControls::GetRecordControl(recording->FileName());
+           if (!rc) {
+              if (recording->Delete()) {
+                 Reply(250, "Recording \"%s\" deleted", Option);
+                 ::Recordings.Load(); // must make sure the global recordings list is updated
+                 }
+              else
+                 Reply(554, "Error while deleting recording!");
+              }
            else
-              Reply(554, "Error while deleting recording!");
+              Reply(550, "Recording \"%s\" is in use by timer %d", Option, rc->Timer()->Index() + 1);
            }
         else
            Reply(550, "Recording \"%s\" not found%s", Option, Recordings.Count() ? "" : " (use LSTR before deleting)");
@@ -518,9 +535,9 @@ void cSVDRP::CmdDELT(const char *Option)
         cTimer *timer = Timers.Get(strtol(Option, NULL, 10) - 1);
         if (timer) {
            if (!timer->Recording()) {
+              isyslog("deleting timer %s", *timer->ToDescr());
               Timers.Del(timer);
-              Timers.Save();
-              isyslog("timer %s deleted", Option);
+              Timers.SetModified();
               Reply(250, "Timer \"%s\" deleted", Option);
               }
            else
@@ -545,8 +562,9 @@ void cSVDRP::CmdGRAB(const char *Option)
      char buf[strlen(Option) + 1];
      char *p = strcpy(buf, Option);
      const char *delim = " \t";
-     FileName = strtok(p, delim);
-     if ((p = strtok(NULL, delim)) != NULL) {
+     char *strtok_next;
+     FileName = strtok_r(p, delim, &strtok_next);
+     if ((p = strtok_r(NULL, delim, &strtok_next)) != NULL) {
         if (strcasecmp(p, "JPEG") == 0)
            Jpeg = true;
         else if (strcasecmp(p, "PNM") == 0)
@@ -556,7 +574,7 @@ void cSVDRP::CmdGRAB(const char *Option)
            return;
            }
         }
-     if ((p = strtok(NULL, delim)) != NULL) {
+     if ((p = strtok_r(NULL, delim, &strtok_next)) != NULL) {
         if (isnumber(p))
            Quality = atoi(p);
         else {
@@ -564,14 +582,14 @@ void cSVDRP::CmdGRAB(const char *Option)
            return;
            }
         }
-     if ((p = strtok(NULL, delim)) != NULL) {
+     if ((p = strtok_r(NULL, delim, &strtok_next)) != NULL) {
         if (isnumber(p))
            SizeX = atoi(p);
         else {
            Reply(501, "Illegal sizex \"%s\"", p);
            return;
            }
-        if ((p = strtok(NULL, delim)) != NULL) {
+        if ((p = strtok_r(NULL, delim, &strtok_next)) != NULL) {
            if (isnumber(p))
               SizeY = atoi(p);
            else {
@@ -584,7 +602,7 @@ void cSVDRP::CmdGRAB(const char *Option)
            return;
            }
         }
-     if ((p = strtok(NULL, delim)) != NULL) {
+     if ((p = strtok_r(NULL, delim, &strtok_next)) != NULL) {
         Reply(501, "Unexpected parameter \"%s\"", p);
         return;
         }
@@ -662,7 +680,7 @@ void cSVDRP::CmdLSTC(const char *Option)
      if (isnumber(Option)) {
         cChannel *channel = Channels.GetByNumber(strtol(Option, NULL, 10));
         if (channel)
-           Reply(250, "%d %s", channel->Number(), channel->ToText());
+           Reply(250, "%d %s", channel->Number(), *channel->ToText());
         else
            Reply(501, "Channel \"%s\" not defined", Option);
         }
@@ -674,7 +692,7 @@ void cSVDRP::CmdLSTC(const char *Option)
               if (channel) {
                  if (strcasestr(channel->Name(), Option)) {
                     if (next)
-                       Reply(-250, "%d %s", next->Number(), next->ToText());
+                       Reply(-250, "%d %s", next->Number(), *next->ToText());
                     next = channel;
                     }
                  }
@@ -685,7 +703,7 @@ void cSVDRP::CmdLSTC(const char *Option)
               i = channel->Number() + 1;
               }
         if (next)
-           Reply(250, "%d %s", next->Number(), next->ToText());
+           Reply(250, "%d %s", next->Number(), *next->ToText());
         else
            Reply(501, "Channel \"%s\" not defined", Option);
         }
@@ -695,7 +713,7 @@ void cSVDRP::CmdLSTC(const char *Option)
      while (i <= Channels.MaxNumber()) {
            cChannel *channel = Channels.GetByNumber(i, 1);
            if (channel)
-              Reply(channel->Number() < Channels.MaxNumber() ? -250 : 250, "%d %s", channel->Number(), channel->ToText());
+              Reply(channel->Number() < Channels.MaxNumber() ? -250 : 250, "%d %s", channel->Number(), *channel->ToText());
            else
               Reply(501, "Channel \"%d\" not found", i);
            i = channel->Number() + 1;
@@ -707,12 +725,69 @@ void cSVDRP::CmdLSTC(const char *Option)
 
 void cSVDRP::CmdLSTE(const char *Option)
 {
-  cMutexLock MutexLock;
-  const cSchedules *Schedules = cSIProcessor::Schedules(MutexLock);
+  cSchedulesLock SchedulesLock;
+  const cSchedules *Schedules = cSchedules::Schedules(SchedulesLock);
   if (Schedules) {
+     const cSchedule* Schedule = NULL;
+     eDumpMode DumpMode = dmAll;
+     time_t AtTime = 0;
+     if (*Option) {
+        char buf[strlen(Option) + 1];
+        strcpy(buf, Option);
+        const char *delim = " \t";
+        char *strtok_next;
+        char *p = strtok_r(buf, delim, &strtok_next);
+        while (p && DumpMode == dmAll) {
+              if (strcasecmp(p, "NOW") == 0)
+                 DumpMode = dmPresent;
+              else if (strcasecmp(p, "NEXT") == 0)
+                 DumpMode = dmFollowing;
+              else if (strcasecmp(p, "AT") == 0) {
+                 DumpMode = dmAtTime;
+                 if ((p = strtok_r(NULL, delim, &strtok_next)) != NULL) {
+                    if (isnumber(p))
+                       AtTime = strtol(p, NULL, 10);
+                    else {
+                       Reply(501, "Invalid time");
+                       return;
+                       }
+                    }
+                 else {
+                    Reply(501, "Missing time");
+                    return;
+                    }
+                 }
+              else if (!Schedule) {
+                 cChannel* Channel = NULL;
+                 if (isnumber(p))
+                    Channel = Channels.GetByNumber(strtol(Option, NULL, 10));
+                 else
+                    Channel = Channels.GetByChannelID(tChannelID::FromString(Option));
+                 if (Channel) {
+                    Schedule = Schedules->GetSchedule(Channel->GetChannelID());
+                    if (!Schedule) {
+                       Reply(550, "No schedule found");
+                       return;
+                       }
+                    }
+                 else {
+                    Reply(550, "Channel \"%s\" not defined", p);
+                    return;
+                    }
+                 }
+              else {
+                 Reply(501, "Unknown option: \"%s\"", p);
+                 return;
+                 }
+              p = strtok_r(NULL, delim, &strtok_next);
+              }
+        }
      FILE *f = fdopen(file, "w");
      if (f) {
-        Schedules->Dump(f, "215-");
+        if (Schedule)
+           Schedule->Dump(f, "215-", DumpMode, AtTime);
+        else
+           Schedules->Dump(f, "215-", DumpMode, AtTime);
         fflush(f);
         Reply(215, "End of EPG data");
         // don't 'fclose(f)' here!
@@ -731,13 +806,15 @@ void cSVDRP::CmdLSTR(const char *Option)
      if (isnumber(Option)) {
         cRecording *recording = Recordings.Get(strtol(Option, NULL, 10) - 1);
         if (recording) {
-           if (recording->Summary()) {
-              char *summary = strdup(recording->Summary());
-              Reply(250, "%s", strreplace(summary,'\n','|'));
-              free(summary);
+           FILE *f = fdopen(file, "w");
+           if (f) {
+              recording->Info()->Write(f, "215-");
+              fflush(f);
+              Reply(215, "End of recording information");
+              // don't 'fclose(f)' here!
               }
            else
-              Reply(550, "No summary availabe");
+              Reply(451, "Can't open file connection");
            }
         else
            Reply(550, "Recording \"%s\" not found", Option);
@@ -762,7 +839,7 @@ void cSVDRP::CmdLSTT(const char *Option)
      if (isnumber(Option)) {
         cTimer *timer = Timers.Get(strtol(Option, NULL, 10) - 1);
         if (timer)
-           Reply(250, "%d %s", timer->Index() + 1, timer->ToText());
+           Reply(250, "%d %s", timer->Index() + 1, *timer->ToText());
         else
            Reply(501, "Timer \"%s\" not defined", Option);
         }
@@ -773,7 +850,7 @@ void cSVDRP::CmdLSTT(const char *Option)
      for (int i = 0; i < Timers.Count(); i++) {
          cTimer *timer = Timers.Get(i);
         if (timer)
-           Reply(i < Timers.Count() - 1 ? -250 : 250, "%d %s", timer->Index() + 1, timer->ToText());
+           Reply(i < Timers.Count() - 1 ? -250 : 250, "%d %s", timer->Index() + 1, *timer->ToText());
         else
            Reply(501, "Timer \"%d\" not found", i + 1);
          }
@@ -806,14 +883,13 @@ void cSVDRP::CmdMODC(const char *Option)
         cChannel *channel = Channels.GetByNumber(n);
         if (channel) {
            cChannel ch;
-           if (ch.Parse(tail, true)) {
+           if (ch.Parse(tail)) {
               if (Channels.HasUniqueChannelID(&ch, channel)) {
                  *channel = ch;
                  Channels.ReNumber();
-                 Channels.Save();
-                 isyslog("modifed channel %d %s", channel->Number(), channel->ToText());
-                 Timers.Save();
-                 Reply(250, "%d %s", channel->Number(), channel->ToText());
+                 Channels.SetModified(true);
+                 isyslog("modifed channel %d %s", channel->Number(), *channel->ToText());
+                 Reply(250, "%d %s", channel->Number(), *channel->ToText());
                  }
               else
                  Reply(501, "Channel settings are not unique");
@@ -842,17 +918,17 @@ void cSVDRP::CmdMODT(const char *Option)
         if (timer) {
            cTimer t = *timer;
            if (strcasecmp(tail, "ON") == 0)
-              t.SetActive(taActive);
+              t.SetFlags(tfActive);
            else if (strcasecmp(tail, "OFF") == 0)
-              t.SetActive(taInactive);
+              t.ClrFlags(tfActive);
            else if (!t.Parse(tail)) {
               Reply(501, "Error in timer settings");
               return;
               }
            *timer = t;
-           Timers.Save();
-           isyslog("timer %d modified (%s)", timer->Index() + 1, timer->Active() ? "active" : "inactive");
-           Reply(250, "%d %s", timer->Index() + 1, timer->ToText());
+           Timers.SetModified();
+           isyslog("timer %s modified (%s)", *timer->ToDescr(), timer->HasFlags(tfActive) ? "active" : "inactive");
+           Reply(250, "%d %s", timer->Index() + 1, *timer->ToText());
            }
         else
            Reply(501, "Timer \"%d\" not defined", n);
@@ -880,15 +956,15 @@ void cSVDRP::CmdNEWC(const char *Option)
 {
   if (*Option) {
      cChannel ch;
-     if (ch.Parse(Option, true)) {
+     if (ch.Parse(Option)) {
         if (Channels.HasUniqueChannelID(&ch)) {
            cChannel *channel = new cChannel;
            *channel = ch;
            Channels.Add(channel);
            Channels.ReNumber();
-           Channels.Save();
-           isyslog("new channel %d %s", channel->Number(), channel->ToText());
-           Reply(250, "%d %s", channel->Number(), channel->ToText());
+           Channels.SetModified(true);
+           isyslog("new channel %d %s", channel->Number(), *channel->ToText());
+           Reply(250, "%d %s", channel->Number(), *channel->ToText());
            }
         else
            Reply(501, "Channel settings are not unique");
@@ -908,13 +984,13 @@ void cSVDRP::CmdNEWT(const char *Option)
         cTimer *t = Timers.GetTimer(timer);
         if (!t) {
            Timers.Add(timer);
-           Timers.Save();
-           isyslog("timer %d added", timer->Index() + 1);
-           Reply(250, "%d %s", timer->Index() + 1, timer->ToText());
+           Timers.SetModified();
+           isyslog("timer %s added", *timer->ToDescr());
+           Reply(250, "%d %s", timer->Index() + 1, *timer->ToText());
            return;
            }
         else
-           Reply(550, "Timer already defined: %d %s", t->Index() + 1, t->ToText());
+           Reply(550, "Timer already defined: %d %s", t->Index() + 1, *t->ToText());
         }
      else
         Reply(501, "Error in timer settings");
@@ -930,11 +1006,8 @@ void cSVDRP::CmdNEXT(const char *Option)
   if (t) {
      time_t Start = t->StartTime();
      int Number = t->Index() + 1;
-     if (!*Option) {
-        char *s = ctime(&Start);
-        s[strlen(s) - 1] = 0; // strip trailing newline
-        Reply(250, "%d %s", Number, s);
-        }
+     if (!*Option)
+        Reply(250, "%d %s", Number, *TimeToString(Start));
      else if (strcasecmp(Option, "ABS") == 0)
         Reply(250, "%d %ld", Number, Start);
      else if (strcasecmp(Option, "REL") == 0)
@@ -955,14 +1028,19 @@ void cSVDRP::CmdPUTE(const char *Option)
      DELETENULL(PUTEhandler);
 }
 
+void cSVDRP::CmdSCAN(const char *Option)
+{
+  EITScanner.ForceScan();
+  Reply(250, "EPG scan triggered");
+}
+
 void cSVDRP::CmdSTAT(const char *Option)
 {
   if (*Option) {
      if (strcasecmp(Option, "DISK") == 0) {
-        int FreeMB;
-        int Percent = VideoDiskSpace(&FreeMB);
-        int Total = (FreeMB / (100 - Percent)) * 100;
-        Reply(250, "%dMB %dMB %d%%", Total, FreeMB, Percent);
+        int FreeMB, UsedMB;
+        int Percent = VideoDiskSpace(&FreeMB, &UsedMB);
+        Reply(250, "%dMB %dMB %d%%", FreeMB + UsedMB, FreeMB, Percent);
         }
      else
         Reply(501, "Invalid Option \"%s\"", Option);
@@ -981,14 +1059,14 @@ void cSVDRP::CmdUPDT(const char *Option)
            t->Parse(Option);
            delete timer;
            timer = t;
-           isyslog("timer %d updated", timer->Index() + 1);
+           isyslog("timer %s updated", *timer->ToDescr());
            }
         else {
            Timers.Add(timer);
-           isyslog("timer %d added", timer->Index() + 1);
+           isyslog("timer %s added", *timer->ToDescr());
            }
-        Timers.Save();
-        Reply(250, "%d %s", timer->Index() + 1, timer->ToText());
+        Timers.SetModified();
+        Reply(250, "%d %s", timer->Index() + 1, *timer->ToText());
         return;
         }
      else
@@ -1063,6 +1141,7 @@ void cSVDRP::Execute(char *Cmd)
   else if (CMD("NEWT"))  CmdNEWT(s);
   else if (CMD("NEXT"))  CmdNEXT(s);
   else if (CMD("PUTE"))  CmdPUTE(s);
+  else if (CMD("SCAN"))  CmdSCAN(s);
   else if (CMD("STAT"))  CmdSTAT(s);
   else if (CMD("UPDT"))  CmdUPDT(s);
   else if (CMD("VOLU"))  CmdVOLU(s);
@@ -1081,7 +1160,7 @@ bool cSVDRP::Process(void)
         char buffer[BUFSIZ];
         gethostname(buffer, sizeof(buffer));
         time_t now = time(NULL);
-        Reply(220, "%s SVDRP VideoDiskRecorder %s; %s", buffer, VDRVERSION, ctime(&now));
+        Reply(220, "%s SVDRP VideoDiskRecorder %s; %s", buffer, VDRVERSION, *TimeToString(now));
         }
      if (NewConnection)
         lastActivity = time(NULL);
