@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: tools.c 1.77 2003/04/06 15:31:45 kls Exp $
+ * $Id: tools.c 1.95 2005/05/29 10:18:26 kls Exp $
  */
 
 #include "tools.h"
@@ -20,6 +20,14 @@
 
 int SysLogLevel = 3;
 
+int BCD2INT(int x)
+{
+  return ((1000000 * BCDCHARTOINT((x >> 24) & 0xFF)) +
+            (10000 * BCDCHARTOINT((x >> 16) & 0xFF)) +
+              (100 * BCDCHARTOINT((x >>  8) & 0xFF)) +
+                     BCDCHARTOINT( x        & 0xFF));
+}
+
 ssize_t safe_read(int filedes, void *buffer, size_t size)
 {
   for (;;) {
@@ -57,16 +65,28 @@ void writechar(int filedes, char c)
   safe_write(filedes, &c, sizeof(c));
 }
 
-char *readline(FILE *f)
+int WriteAllOrNothing(int fd, const uchar *Data, int Length, int TimeoutMs, int RetryMs)
 {
-  static char buffer[MAXPARSEBUFFER];
-  if (fgets(buffer, sizeof(buffer), f) > 0) {
-     int l = strlen(buffer) - 1;
-     if (l >= 0 && buffer[l] == '\n')
-        buffer[l] = 0;
-     return buffer;
-     }
-  return NULL;
+  int written = 0;
+  while (Length > 0) {
+        int w = write(fd, Data + written, Length);
+        if (w > 0) {
+           Length -= w;
+           written += w;
+           }
+        else if (written > 0 && !FATALERRNO) {
+           // we've started writing, so we must finish it!
+           cTimeMs t;
+           cPoller Poller(fd, true);
+           Poller.Poll(RetryMs);
+           if (TimeoutMs > 0 && (TimeoutMs -= t.Elapsed()) <= 0)
+              break;
+           }
+        else
+           // nothing written yet (or fatal error), so we can just return the error code:
+           return w;
+        }
+  return written;
 }
 
 char *strcpyrealloc(char *dest, const char *src)
@@ -159,15 +179,15 @@ char *compactspace(char *s)
   return s;
 }
 
-const char *strescape(const char *s, const char *chars)
+cString strescape(const char *s, const char *chars)
 {
-  static char *buffer = NULL;
+  char *buffer;
   const char *p = s;
   char *t = NULL;
   while (*p) {
         if (strchr(chars, *p)) {
            if (!t) {
-              buffer = (char *)realloc(buffer, 2 * strlen(s) + 1);
+              buffer = MALLOC(char, 2 * strlen(s) + 1);
               t = buffer + (p - s);
               s = strcpy(buffer, s);
               }
@@ -179,7 +199,7 @@ const char *strescape(const char *s, const char *chars)
         }
   if (t)
      *t = 0;
-  return s;
+  return cString(s, t != NULL);
 }
 
 bool startswith(const char *s, const char *p)
@@ -214,25 +234,6 @@ int numdigits(int n)
   return strlen(buf);
 }
 
-int time_ms(void)
-{
-  static time_t t0 = 0;
-  struct timeval t;
-  if (gettimeofday(&t, NULL) == 0) {
-     if (t0 == 0)
-        t0 = t.tv_sec; // this avoids an overflow (we only work with deltas)
-     return (t.tv_sec - t0) * 1000 + t.tv_usec / 1000;
-     }
-  return 0;
-}
-
-void delay_ms(int ms)
-{
-  int t0 = time_ms();
-  while (time_ms() - t0 < ms)
-        ;
-}
-
 bool isnumber(const char *s)
 {
   if (!*s)
@@ -245,18 +246,17 @@ bool isnumber(const char *s)
   return true;
 }
 
-const char *itoa(int n)
+cString AddDirectory(const char *DirName, const char *FileName)
 {
-  static char buf[16];
-  snprintf(buf, sizeof(buf), "%d", n);
-  return buf;
+  char *buf;
+  asprintf(&buf, "%s/%s", DirName && *DirName ? DirName : ".", FileName);
+  return cString(buf, true);
 }
 
-const char *AddDirectory(const char *DirName, const char *FileName)
+cString itoa(int n)
 {
-  static char *buf = NULL;
-  free(buf);
-  asprintf(&buf, "%s/%s", DirName && *DirName ? DirName : ".", FileName);
+  char buf[16];
+  snprintf(buf, sizeof(buf), "%d", n);
   return buf;
 }
 
@@ -328,10 +328,10 @@ bool RemoveFileOrDir(const char *FileName, bool FollowSymlinks)
   struct stat st;
   if (stat(FileName, &st) == 0) {
      if (S_ISDIR(st.st_mode)) {
-        DIR *d = opendir(FileName);
-        if (d) {
+        cReadDir d(FileName);
+        if (d.Ok()) {
            struct dirent *e;
-           while ((e = readdir(d)) != NULL) {
+           while ((e = d.Next()) != NULL) {
                  if (strcmp(e->d_name, ".") && strcmp(e->d_name, "..")) {
                     char *buffer;
                     asprintf(&buffer, "%s/%s", FileName, e->d_name);
@@ -359,7 +359,6 @@ bool RemoveFileOrDir(const char *FileName, bool FollowSymlinks)
                     free(buffer);
                     }
                  }
-           closedir(d);
            }
         else {
            LOG_ERROR_STR(FileName);
@@ -381,11 +380,11 @@ bool RemoveFileOrDir(const char *FileName, bool FollowSymlinks)
 
 bool RemoveEmptyDirectories(const char *DirName, bool RemoveThis)
 {
-  DIR *d = opendir(DirName);
-  if (d) {
+  cReadDir d(DirName);
+  if (d.Ok()) {
      bool empty = true;
      struct dirent *e;
-     while ((e = readdir(d)) != NULL) {
+     while ((e = d.Next()) != NULL) {
            if (strcmp(e->d_name, ".") && strcmp(e->d_name, "..") && strcmp(e->d_name, "lost+found")) {
               char *buffer;
               asprintf(&buffer, "%s/%s", DirName, e->d_name);
@@ -406,7 +405,6 @@ bool RemoveEmptyDirectories(const char *DirName, bool RemoveThis)
               free(buffer);
               }
            }
-     closedir(d);
      if (RemoveThis && empty) {
         dsyslog("removing %s", DirName);
         if (remove(DirName) < 0) {
@@ -443,7 +441,7 @@ char *ReadLink(const char *FileName)
 
 bool SpinUpDisk(const char *FileName)
 {
-  static char *buf = NULL;
+  char *buf = NULL;
   for (int n = 0; n < 10; n++) {
       free(buf);
       if (DirectoryOk(FileName))
@@ -456,48 +454,155 @@ bool SpinUpDisk(const char *FileName)
          int f = open(buf, O_WRONLY | O_CREAT, S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH);
          // O_SYNC doesn't work on all file systems
          if (f >= 0) {
+            if (fdatasync(f) < 0)
+               LOG_ERROR_STR(buf);
             close(f);
-            system("sync");
             remove(buf);
             gettimeofday(&tp2, NULL);
             double seconds = (((long long)tp2.tv_sec * 1000000 + tp2.tv_usec) - ((long long)tp1.tv_sec * 1000000 + tp1.tv_usec)) / 1000000.0;
             if (seconds > 0.5)
                dsyslog("SpinUpDisk took %.2f seconds\n", seconds);
+            free(buf);
             return true;
             }
          else
             LOG_ERROR_STR(buf);
          }
       }
+  free(buf);
   esyslog("ERROR: SpinUpDisk failed");
   return false;
 }
 
-const char *WeekDayName(int WeekDay)
+time_t LastModifiedTime(const char *FileName)
 {
-  static char buffer[4];
+  struct stat fs;
+  if (stat(FileName, &fs) == 0)
+     return fs.st_mtime;
+  return 0;
+}
+
+// --- cTimeMs ---------------------------------------------------------------
+
+cTimeMs::cTimeMs(void)
+{
+  Set();
+}
+
+uint64 cTimeMs::Now(void)
+{
+  struct timeval t;
+  if (gettimeofday(&t, NULL) == 0)
+     return (uint64(t.tv_sec)) * 1000 + t.tv_usec / 1000;
+  return 0;
+}
+
+void cTimeMs::Set(int Ms)
+{
+  begin = Now() + Ms;
+}
+
+bool cTimeMs::TimedOut(void)
+{
+  return Now() >= begin;
+}
+
+uint64 cTimeMs::Elapsed(void)
+{
+  return Now() - begin;
+}
+
+// --- cString ---------------------------------------------------------------
+
+cString::cString(const char *S, bool TakePointer)
+{
+  s = TakePointer ? (char *)S : S ? strdup(S) : NULL;
+}
+
+cString::~cString()
+{
+  free(s);
+}
+
+cString &cString::operator=(const cString &String)
+{
+  s = String.s ? strdup(String.s) : NULL;
+  return *this;
+}
+
+cString WeekDayName(int WeekDay)
+{
+  char buffer[4];
   WeekDay = WeekDay == 0 ? 6 : WeekDay - 1; // we start with monday==0!
   if (0 <= WeekDay && WeekDay <= 6) {
      const char *day = tr("MonTueWedThuFriSatSun");
      day += WeekDay * 3;
-     strncpy(buffer, day, 3);
+     strn0cpy(buffer, day, sizeof(buffer));
      return buffer;
      }
   else
      return "???";
 }
 
-const char *DayDateTime(time_t t)
+cString WeekDayName(time_t t)
 {
-  static char buffer[32];
+  struct tm tm_r;
+  return WeekDayName(localtime_r(&t, &tm_r)->tm_wday);
+}
+
+cString DayDateTime(time_t t)
+{
+  char buffer[32];
   if (t == 0)
      time(&t);
   struct tm tm_r;
   tm *tm = localtime_r(&t, &tm_r);
-  snprintf(buffer, sizeof(buffer), "%s %2d.%02d %02d:%02d", WeekDayName(tm->tm_wday), tm->tm_mday, tm->tm_mon + 1, tm->tm_hour, tm->tm_min);
+  snprintf(buffer, sizeof(buffer), "%s %2d.%02d %02d:%02d", *WeekDayName(tm->tm_wday), tm->tm_mday, tm->tm_mon + 1, tm->tm_hour, tm->tm_min);
   return buffer;
 }
 
+cString TimeToString(time_t t)
+{
+  char buffer[32];
+  if (ctime_r(&t, buffer)) {
+     buffer[strlen(buffer) - 1] = 0; // strip trailing newline
+     return buffer;
+     }
+  return "???";
+}
+
+cString DateString(time_t t)
+{
+  char buf[32];
+  struct tm tm_r;
+  tm *tm = localtime_r(&t, &tm_r);
+  char *p = stpcpy(buf, WeekDayName(tm->tm_wday));
+  *p++ = ' ';
+  strftime(p, sizeof(buf) - (p - buf), "%d.%m.%Y", tm);
+  return buf;
+}
+
+cString TimeString(time_t t)
+{
+  char buf[25];
+  struct tm tm_r;
+  strftime(buf, sizeof(buf), "%R", localtime_r(&t, &tm_r));
+  return buf;
+}
+
+// --- cReadLine -------------------------------------------------------------
+
+char *cReadLine::Read(FILE *f)
+{
+  if (fgets(buffer, sizeof(buffer), f) > 0) {
+     int l = strlen(buffer) - 1;
+     if (l >= 0 && buffer[l] == '\n')
+        buffer[l] = 0;
+     return buffer;
+     }
+  return NULL;
+}
+
 // --- cPoller ---------------------------------------------------------------
 
 cPoller::cPoller(int FileHandle, bool Out)
@@ -510,12 +615,13 @@ bool cPoller::Add(int FileHandle, bool Out)
 {
   if (FileHandle >= 0) {
      for (int i = 0; i < numFileHandles; i++) {
-         if (pfd[i].fd == FileHandle)
+         if (pfd[i].fd == FileHandle && pfd[i].events == (Out ? POLLOUT : POLLIN))
             return true;
          }
      if (numFileHandles < MaxPollFiles) {
         pfd[numFileHandles].fd = FileHandle;
         pfd[numFileHandles].events = Out ? POLLOUT : POLLIN;
+        pfd[numFileHandles].revents = 0;
         numFileHandles++;
         return true;
         }
@@ -534,6 +640,24 @@ bool cPoller::Poll(int TimeoutMs)
   return false;
 }
 
+// --- cReadDir --------------------------------------------------------------
+
+cReadDir::cReadDir(const char *Directory)
+{
+  directory = opendir(Directory);
+}
+
+cReadDir::~cReadDir()
+{
+  if (directory)
+     closedir(directory);
+}
+
+struct dirent *cReadDir::Next(void)
+{
+  return directory && readdir_r(directory, &u.d, &result) == 0 ? result : NULL;
+}
+
 // --- cFile -----------------------------------------------------------------
 
 bool cFile::files[FD_SETSIZE] = { false };
@@ -791,7 +915,7 @@ void cListObject::Unlink(void)
   next = prev = NULL;
 }
 
-int cListObject::Index(void)
+int cListObject::Index(void) const
 {
   cListObject *p = prev;
   int i = 0;
@@ -808,6 +932,7 @@ int cListObject::Index(void)
 cListBase::cListBase(void)
 { 
   objects = lastObject = NULL;
+  count = 0;
 }
 
 cListBase::~cListBase()
@@ -828,6 +953,7 @@ void cListBase::Add(cListObject *Object, cListObject *After)
         objects = Object;
      lastObject = Object;
      }
+  count++;
 }
 
 void cListBase::Ins(cListObject *Object, cListObject *Before)
@@ -843,6 +969,7 @@ void cListBase::Ins(cListObject *Object, cListObject *Before)
         lastObject = Object;
      objects = Object;
      }
+  count++;
 }
 
 void cListBase::Del(cListObject *Object, bool DeleteObject)
@@ -854,6 +981,7 @@ void cListBase::Del(cListObject *Object, bool DeleteObject)
   Object->Unlink();
   if (DeleteObject)
      delete Object;
+  count--;
 }
 
 void cListBase::Move(int From, int To)
@@ -893,6 +1021,7 @@ void cListBase::Clear(void)
         objects = object;
         }
   objects = lastObject = NULL;
+  count = 0;
 }
 
 cListObject *cListBase::Get(int Index) const
@@ -905,31 +1034,81 @@ cListObject *cListBase::Get(int Index) const
   return object;
 }
 
-int cListBase::Count(void) const
+static int CompareListObjects(const void *a, const void *b)
 {
-  int n = 0;
-  cListObject *object = objects;
-
-  while (object) {
-        n++;
-        object = object->Next();
-        }
-  return n;
+  const cListObject *la = *(const cListObject **)a;
+  const cListObject *lb = *(const cListObject **)b;
+  return la->Compare(*lb);
 }
 
 void cListBase::Sort(void)
 {
-  bool swapped;
-  do {
-     swapped = false;
-     cListObject *object = objects;
-     while (object) {
-           if (object->Next() && *object->Next() < *object) {
-              Move(object->Next(), object);
-              swapped = true;
-              }
-           object = object->Next();
-           }
-     } while (swapped);
+  int n = Count();
+  cListObject *a[n];
+  cListObject *object = objects;
+  int i = 0;
+  while (object && i < n) {
+        a[i++] = object;
+        object = object->Next();
+        }
+  qsort(a, n, sizeof(cListObject *), CompareListObjects);
+  objects = lastObject = NULL;
+  for (i = 0; i < n; i++) {
+      a[i]->Unlink();
+      count--;
+      Add(a[i]);
+      }
 }
 
+// --- cHashBase -------------------------------------------------------------
+
+cHashBase::cHashBase(int Size)
+{
+  size = Size;
+  hashTable = (cList<cHashObject>**)calloc(size, sizeof(cList<cHashObject>*));
+}
+
+cHashBase::~cHashBase(void)
+{
+  for (int i = 0; i < size; i++)
+      delete hashTable[i];
+  free(hashTable);
+}
+
+void cHashBase::Add(cListObject *Object, unsigned int Id)
+{
+  unsigned int hash = hashfn(Id);
+  if (!hashTable[hash])
+     hashTable[hash] = new cList<cHashObject>;
+  hashTable[hash]->Add(new cHashObject(Object, Id));
+}
+
+void cHashBase::Del(cListObject *Object, unsigned int Id)
+{
+  cList<cHashObject> *list = hashTable[hashfn(Id)];
+  if (list) {
+     for (cHashObject *hob = list->First(); hob; hob = list->Next(hob)) {
+         if (hob->object == Object) {
+            list->Del(hob);
+            break;
+            }
+         }
+     }
+}
+
+cListObject *cHashBase::Get(unsigned int Id) const
+{
+  cList<cHashObject> *list = hashTable[hashfn(Id)];
+  if (list) {
+     for (cHashObject *hob = list->First(); hob; hob = list->Next(hob)) {
+         if (hob->id == Id)
+            return hob->object;
+         }
+     }
+  return NULL;
+}
+
+cList<cHashObject> *cHashBase::GetList(unsigned int Id) const
+{
+  return hashTable[hashfn(Id)];
+}
