@@ -22,7 +22,7 @@
  *
  * The project's page is at http://www.cadsoft.de/vdr
  *
- * $Id: vdr.c 1.169 2003/09/14 09:36:54 kls Exp $
+ * $Id: vdr.c 1.208 2005/06/18 11:19:07 kls Exp $
  */
 
 #include <getopt.h>
@@ -39,24 +39,32 @@
 #include "diseqc.h"
 #include "dvbdevice.h"
 #include "eitscan.h"
+#include "epg.h"
 #include "i18n.h"
 #include "interface.h"
 #include "keys.h"
 #include "lirc.h"
 #include "menu.h"
-#include "osd.h"
+#include "osdbase.h"
 #include "plugin.h"
 #include "rcu.h"
 #include "recording.h"
+#include "skinclassic.h"
+#include "skinsttng.h"
 #include "sources.h"
+#include "themes.h"
 #include "timers.h"
 #include "tools.h"
+#include "transfer.h"
 #include "videodir.h"
 
-#define MINCHANNELWAIT  10 // seconds to wait between failed channel switchings
-#define ACTIVITYTIMEOUT 60 // seconds before starting housekeeping
-#define SHUTDOWNWAIT   300 // seconds to wait in user prompt before automatic shutdown
-#define MANUALSTART    600 // seconds the next timer must be in the future to assume manual start
+#define MINCHANNELWAIT     10 // seconds to wait between failed channel switchings
+#define ACTIVITYTIMEOUT    60 // seconds before starting housekeeping
+#define SHUTDOWNWAIT      300 // seconds to wait in user prompt before automatic shutdown
+#define MANUALSTART       600 // seconds the next timer must be in the future to assume manual start
+#define CHANNELSAVEDELTA  600 // seconds before saving channels.conf after automatic modifications
+
+#define EXIT(v) { ExitCode = (v); goto Exit; }
 
 static int Interrupted = 0;
 
@@ -93,10 +101,12 @@ int main(int argc, char *argv[])
 #define DEFAULTSVDRPPORT 2001
 #define DEFAULTWATCHDOG     0 // seconds
 #define DEFAULTPLUGINDIR PLUGINDIR
+#define DEFAULTEPGDATAFILENAME "epg.data"
 
   int SVDRPport = DEFAULTSVDRPPORT;
   const char *AudioCommand = NULL;
   const char *ConfigDirectory = NULL;
+  const char *EpgDataFileName = DEFAULTEPGDATAFILENAME;
   bool DisplayHelp = false;
   bool DisplayVersion = false;
   bool DaemonMode = false;
@@ -106,6 +116,7 @@ int main(int argc, char *argv[])
   const char *Terminal = NULL;
   const char *Shutdown = NULL;
   cPluginManager PluginManager(DEFAULTPLUGINDIR);
+  int ExitCode = 0;
 
   static struct option long_options[] = {
       { "audio",    required_argument, NULL, 'a' },
@@ -146,7 +157,7 @@ int main(int argc, char *argv[])
                     fprintf(stderr, "vdr: invalid DVB device number: %s\n", optarg);
                     return 2;
                     break;
-          case 'E': cSIProcessor::SetEpgDataFileName(*optarg != '-' ? optarg : NULL);
+          case 'E': EpgDataFileName = (*optarg != '-' ? optarg : NULL);
                     break;
           case 'h': DisplayHelp = true;
                     break;
@@ -239,7 +250,8 @@ int main(int argc, char *argv[])
                "                           there may be several -D options (default: all DVB\n"
                "                           devices will be used)\n"
                "  -E FILE   --epgfile=FILE write the EPG data into the given FILE (default is\n"
-               "                           %s); use '-E-' to disable this\n"
+               "                           '%s' in the video directory)\n"
+               "                           '-E-' disables this\n"
                "                           if FILE is a directory, the default EPG file will be\n"
                "                           created in that directory\n"
                "  -h,       --help         print this help and exit\n"
@@ -261,7 +273,7 @@ int main(int argc, char *argv[])
                "  -w SEC,   --watchdog=SEC activate the watchdog timer with a timeout of SEC\n"
                "                           seconds (default: %d); '0' disables the watchdog\n"
                "\n",
-               cSIProcessor::GetEpgDataFileName() ? cSIProcessor::GetEpgDataFileName() : "'-'",
+               DEFAULTEPGDATAFILENAME,
                DEFAULTPLUGINDIR,
                DEFAULTSVDRPPORT,
                VideoDirectory,
@@ -290,6 +302,14 @@ int main(int argc, char *argv[])
      return 0;
      }
 
+  // Check for UTF-8 and exit if present - asprintf() will fail if it encounters 8 bit ASCII codes
+  char *LangEnv;
+  if ((LangEnv = getenv("LANG"))     != NULL && strcasestr(LangEnv, "utf") ||
+      (LangEnv = getenv("LC_CTYPE")) != NULL && strcasestr(LangEnv, "utf")) {
+     fprintf(stderr, "vdr: please turn off UTF-8 before starting VDR\n");
+     return 2;
+     }
+
   // Log file:
 
   if (SysLogLevel > 0)
@@ -305,7 +325,6 @@ int main(int argc, char *argv[])
   // Daemon mode:
 
   if (DaemonMode) {
-#if !defined(DEBUG_OSD)
      pid_t pid = fork();
      if (pid < 0) {
         fprintf(stderr, "%m\n");
@@ -317,10 +336,6 @@ int main(int argc, char *argv[])
      fclose(stdin);
      fclose(stdout);
      fclose(stderr);
-#else
-     fprintf(stderr, "vdr: can't run in daemon mode with DEBUG_OSD on!\n");
-     return 2;
-#endif
      }
   else if (Terminal) {
      // Claim new controlling terminal
@@ -332,10 +347,25 @@ int main(int argc, char *argv[])
 
   isyslog("VDR version %s started", VDRVERSION);
 
+  // Main program loop variables - need to be here to have them initialized before any EXIT():
+
+  cOsdObject *Menu = NULL;
+  cOsdObject *Temp = NULL;
+  int LastChannel = -1;
+  int LastTimerChannel = -1;
+  int PreviousChannel[2] = { 1, 1 };
+  int PreviousChannelIndex = 0;
+  time_t LastChannelChanged = time(NULL);
+  time_t LastActivity = 0;
+  int MaxLatencyTime = 0;
+  bool ForceShutdown = false;
+  bool UserShutdown = false;
+  bool TimerInVpsMargin = false;
+
   // Load plugins:
 
   if (!PluginManager.LoadPlugins(true))
-     return 2;
+     EXIT(2);
 
   // Configuration data:
 
@@ -343,6 +373,7 @@ int main(int argc, char *argv[])
      ConfigDirectory = VideoDirectory;
 
   cPlugin::SetConfigDirectory(ConfigDirectory);
+  cThemes::SetThemesDirectory(AddDirectory(ConfigDirectory, "themes"));
 
   Setup.Load(AddDirectory(ConfigDirectory, "setup.conf"));
   if (!(Sources.Load(AddDirectory(ConfigDirectory, "sources.conf"), true, true) &&
@@ -356,7 +387,26 @@ int main(int argc, char *argv[])
         Keys.Load(AddDirectory(ConfigDirectory, "remote.conf")) &&
         KeyMacros.Load(AddDirectory(ConfigDirectory, "keymacros.conf"), true)
         ))
-     return 2;
+     EXIT(2);
+
+  cFont::SetCode(I18nCharSets()[Setup.OSDLanguage]);
+
+  // EPG data:
+
+  if (EpgDataFileName) {
+     const char *EpgDirectory = NULL;
+     if (DirectoryOk(EpgDataFileName)) {
+        EpgDirectory = EpgDataFileName;
+        EpgDataFileName = DEFAULTEPGDATAFILENAME;
+        }
+     else if (*EpgDataFileName != '/' && *EpgDataFileName != '.')
+        EpgDirectory = VideoDirectory;
+     if (EpgDirectory)
+        cSchedules::SetEpgDataFileName(AddDirectory(EpgDirectory, EpgDataFileName));
+     else
+        cSchedules::SetEpgDataFileName(EpgDataFileName);
+     cSchedules::Read();
+     }
 
   // DVB interfaces:
 
@@ -365,7 +415,7 @@ int main(int argc, char *argv[])
   // Initialize plugins:
 
   if (!PluginManager.InitializePlugins())
-     return 2;
+     EXIT(2);
 
   // Primary device:
 
@@ -388,28 +438,36 @@ int main(int argc, char *argv[])
         fprintf(stderr, "vdr: %s\n", msg);
         esyslog("ERROR: %s", msg);
         if (!cDevice::SetPrimaryDevice(1))
-           return 2;
+           EXIT(2);
         if (!cDevice::PrimaryDevice()) {
            const char *msg = "no primary device found - giving up!";
            fprintf(stderr, "vdr: %s\n", msg);
            esyslog("ERROR: %s", msg);
-           return 2;
+           EXIT(2);
            }
         }
      }
 
-  // OSD:
-
-  cOsd::Initialize();
-
   // User interface:
 
   Interface = new cInterface(SVDRPport);
 
+  // Default skins:
+
+  new cSkinClassic;
+  new cSkinSTTNG;
+  Skins.SetCurrent(Setup.OSDSkin);
+  cThemes::Load(Skins.Current()->Name(), Setup.OSDTheme, Skins.Current()->Theme());
+
   // Start plugins:
 
   if (!PluginManager.StartPlugins())
-     return 2;
+     EXIT(2);
+
+  // Set skin and theme in case they're implemented by a plugin:
+
+  Skins.SetCurrent(Setup.OSDSkin);
+  cThemes::Load(Skins.Current()->Name(), Setup.OSDTheme, Skins.Current()->Theme());
 
   // Remote Controls:
 #if defined(REMOTE_RCU)
@@ -436,7 +494,9 @@ int main(int argc, char *argv[])
   else
      cDevice::PrimaryDevice()->SetVolume(Setup.CurrentVolume, true);
 
-  cSIProcessor::Read();
+  // Recordings:
+
+  Recordings.Load();
 
   // Signal handlers:
 
@@ -456,35 +516,30 @@ int main(int argc, char *argv[])
 
   // Main program loop:
 
-  cOsdObject *Menu = NULL;
-  cOsdObject *Temp = NULL;
-  int LastChannel = -1;
-  int LastTimerChannel = -1;
-  int PreviousChannel[2] = { 1, 1 };
-  int PreviousChannelIndex = 0;
-  time_t LastChannelChanged = time(NULL);
-  time_t LastActivity = 0;
-  int MaxLatencyTime = 0;
-  bool ForceShutdown = false;
-  bool UserShutdown = false;
-
   while (!Interrupted) {
         // Handle emergency exits:
         if (cThread::EmergencyExit()) {
            esyslog("emergency exit requested - shutting down");
            break;
            }
+#ifdef DEBUGRINGBUFFERS
+        cRingBufferLinear::PrintDebugRBL();
+#endif
         // Attach launched player control:
         cControl::Attach();
         // Make sure we have a visible programme in case device usage has changed:
         if (!EITScanner.Active() && cDevice::PrimaryDevice()->HasDecoder() && !cDevice::PrimaryDevice()->HasProgramme()) {
            static time_t lastTime = 0;
            if (time(NULL) - lastTime > MINCHANNELWAIT) {
-              if (!Channels.SwitchTo(cDevice::CurrentChannel()) // try to switch to the original channel...
-                  && !(LastTimerChannel > 0 && Channels.SwitchTo(LastTimerChannel)) // ...or the one used by the last timer...
-                  && !cDevice::SwitchChannel(1) // ...or the next higher available one...
-                  && !cDevice::SwitchChannel(-1)) // ...or the next lower available one
-                 lastTime = time(NULL); // don't do this too often
+              cChannel *Channel = Channels.GetByNumber(cDevice::CurrentChannel());
+              if (Channel && (Channel->Vpid() || Channel->Apid(0))) {
+                 if (!Channels.SwitchTo(cDevice::CurrentChannel()) // try to switch to the original channel...
+                     && !(LastTimerChannel > 0 && Channels.SwitchTo(LastTimerChannel)) // ...or the one used by the last timer...
+                     && !cDevice::SwitchChannel(1) // ...or the next higher available one...
+                     && !cDevice::SwitchChannel(-1)) // ...or the next lower available one
+                    ; 
+                 }   
+              lastTime = time(NULL); // don't do this too often
               LastTimerChannel = -1;
               }
            }
@@ -496,6 +551,39 @@ int main(int argc, char *argv[])
               dsyslog("max. latency time %d seconds", MaxLatencyTime);
               }
            }
+        // Handle channel and timer modifications:
+        if (!Channels.BeingEdited() && !Timers.BeingEdited()) {
+           int modified = Channels.Modified();
+           static time_t ChannelSaveTimeout = 0;
+           // Channels and timers need to be stored in a consistent manner,
+           // therefore if one of them is changed, we save both.
+           if (modified == CHANNELSMOD_USER || Timers.Modified())
+              ChannelSaveTimeout = 1; // triggers an immediate save
+           else if (modified && !ChannelSaveTimeout)
+              ChannelSaveTimeout = time(NULL) + CHANNELSAVEDELTA;
+           bool timeout = ChannelSaveTimeout == 1 || ChannelSaveTimeout && time(NULL) > ChannelSaveTimeout && !cRecordControls::Active();
+           if ((modified || timeout) && Channels.Lock(false, 100)) {
+              if (timeout) {
+                 Channels.Save();
+                 Timers.Save();
+                 ChannelSaveTimeout = 0;
+                 }
+              for (cChannel *Channel = Channels.First(); Channel; Channel = Channels.Next(Channel)) {
+                  if (Channel->Modification(CHANNELMOD_RETUNE)) {
+                     cRecordControls::ChannelDataModified(Channel);
+                     if (Channel->Number() == cDevice::CurrentChannel()) {
+                        if (!cDevice::PrimaryDevice()->Replaying() || cTransferControl::ReceiverDevice()) {
+                           if (cDevice::ActualDevice()->ProvidesTransponder(Channel)) { // avoids retune on devices that don't really access the transponder
+                              isyslog("retuning due to modification of channel %d", Channel->Number());
+                              Channels.SwitchTo(Channel->Number());
+                              }
+                           }
+                        }
+                     }
+                  }
+              Channels.Unlock();
+              }
+           }
         // Channel display:
         if (!EITScanner.Active() && cDevice::CurrentChannel() != LastChannel) {
            if (!Menu)
@@ -507,8 +595,15 @@ int main(int argc, char *argv[])
            PreviousChannel[PreviousChannelIndex ^= 1] = LastChannel;
         // Timers and Recordings:
         if (!Timers.BeingEdited()) {
-           time_t Now = time(NULL); // must do both following calls with the exact same time!
+           // Delete expired timers:
+           Timers.DeleteExpired();
+           // Assign events to timers:
+           Timers.SetEvents();
+           // Must do all following calls with the exact same time!
+           time_t Now = time(NULL);
+           // Process ongoing recordings:
            cRecordControls::Process(Now);
+           // Start new recordings:
            cTimer *Timer = Timers.GetMatch(Now);
            if (Timer) {
               if (!cRecordControls::Start(Timer))
@@ -516,9 +611,27 @@ int main(int argc, char *argv[])
               else
                  LastTimerChannel = Timer->Channel()->Number();
               }
+           // Make sure VPS timers "see" their channel early enough:
+           TimerInVpsMargin = false;
+           for (cTimer *Timer = Timers.First(); Timer; Timer = Timers.Next(Timer)) {
+               if (Timer->HasFlags(tfActive | tfVps) && !Timer->Recording() && !Timer->Pending() && Timer->Matches(Now + Setup.VpsMargin, true)) {
+                  if (!Timer->InVpsMargin()) {
+                     Timer->SetInVpsMargin(true);
+                     //XXX if not primary device has TP???
+                     LastTimerChannel = Timer->Channel()->Number();
+                     cRecordControls::Start(Timer); // will only switch the device
+                     }
+                  }
+               else
+                  Timer->SetInVpsMargin(false);
+               if (Timer->InVpsMargin())
+                  TimerInVpsMargin = true;
+               }
            }
+        if (!Menu && Recordings.NeedsUpdate())
+           Recordings.Load();
         // CAM control:
-        if (!Menu && !Interface->IsOpen())
+        if (!Menu && !cOsd::IsOpen())
            Menu = CamControl();
         // User Input:
         cOsdObject *Interact = Menu ? Menu : cControl::Control();
@@ -564,7 +677,7 @@ int main(int argc, char *argv[])
           case kChanDn:
                cDevice::SwitchChannel(NORMALKEY(key) == kChanUp ? 1 : -1);
                break;
-          // Volume Control:
+          // Volume control:
           case kVolUp|k_Repeat:
           case kVolUp:
           case kVolDn|k_Repeat:
@@ -578,18 +691,32 @@ int main(int argc, char *argv[])
                   }
                else
                   cDevice::PrimaryDevice()->SetVolume(NORMALKEY(key) == kVolDn ? -VOLUMEDELTA : VOLUMEDELTA);
-               if (!Menu && !Interface->IsOpen())
+               if (!Menu && !cOsd::IsOpen())
                   Menu = Temp = cDisplayVolume::Create();
                cDisplayVolume::Process(key);
                key = kNone; // nobody else needs to see these keys
                break;
+          // Audio track control:
+          case kAudio:
+               if (cControl::Control())
+                  cControl::Control()->Hide();
+               if (Temp && !cDisplayTracks::IsOpen()) {
+                  DELETENULL(Menu);
+                  Temp = NULL;
+                  }
+               if (!Menu && !cOsd::IsOpen())
+                  Menu = Temp = cDisplayTracks::Create();
+               else
+                  cDisplayTracks::Process(key);
+               key = kNone;
+               break;
           // Pausing live video:
           case kPause:
                if (!cControl::Control()) {
                   DELETENULL(Menu);
                   Temp = NULL;
                   if (!cRecordControls::PauseLiveVideo())
-                     Interface->Error(tr("No free DVB device to record!"));
+                     Skins.Message(mtError, tr("No free DVB device to record!"));
                   key = kNone; // nobody else needs to see this key
                   }
                break;
@@ -597,19 +724,18 @@ int main(int argc, char *argv[])
           case kRecord:
                if (!cControl::Control()) {
                   if (cRecordControls::Start())
-                     ;//XXX Interface->Info(tr("Recording"));
+                     ;//XXX Skins.Message(mtInfo, tr("Recording"));
                   else
-                     Interface->Error(tr("No free DVB device to record!"));
+                     Skins.Message(mtError, tr("No free DVB device to record!"));
                   key = kNone; // nobody else needs to see this key
                   }
                break;
           // Power off:
           case kPower: isyslog("Power button pressed");
                        DELETENULL(Menu);
-                       cControl::Shutdown();
                        Temp = NULL;
                        if (!Shutdown) {
-                          Interface->Error(tr("Can't shutdown - option '-s' not given!"));
+                          Skins.Message(mtError, tr("Can't shutdown - option '-s' not given!"));
                           break;
                           }
                        if (cRecordControls::Active()) {
@@ -631,14 +757,14 @@ int main(int argc, char *argv[])
                             cControl::Shutdown(); // just in case
                             Temp = NULL;
                             if (!cRecordControls::PauseLiveVideo())
-                               Interface->Error(tr("No free DVB device to record!"));
+                               Skins.Message(mtError, tr("No free DVB device to record!"));
                             break;
              case osRecord: DELETENULL(Menu);
                             Temp = NULL;
                             if (cRecordControls::Start())
-                               ;//XXX Interface->Info(tr("Recording"));
+                               ;//XXX Skins.Message(mtInfo, tr("Recording"));
                             else
-                               Interface->Error(tr("No free DVB device to record!"));
+                               Skins.Message(mtError, tr("No free DVB device to record!"));
                             break;
              case osRecordings:
                             DELETENULL(Menu);
@@ -660,7 +786,7 @@ int main(int argc, char *argv[])
                             DELETENULL(Menu);
                             cControl::Shutdown();
                             Temp = NULL;
-                            Interface->Info(tr("Switching primary DVB..."));
+                            Skins.Message(mtInfo, tr("Switching primary DVB..."));
                             cDevice::SetPrimaryDevice(Setup.PrimaryDVB);
                             break;
              case osPlugin: DELETENULL(Menu);
@@ -708,6 +834,14 @@ int main(int argc, char *argv[])
                   break;
              // Viewing Control:
              case kOk:   LastChannel = -1; break; // forces channel display
+             // Instant resume of the last viewed recording:
+             case kPlay:
+                  if (cReplayControl::LastReplayed()) {
+                     cControl::Shutdown();
+                     Temp = NULL;
+                     cControl::Launch(new cReplayControl);
+                     }
+                  break;
              // Key macros:
              case kRed:
              case kGreen:
@@ -717,12 +851,13 @@ int main(int argc, char *argv[])
              }
            }
         if (!Menu) {
-           EITScanner.Process();
+           if (!TimerInVpsMargin)
+              EITScanner.Process();
            if (!cCutter::Active() && cCutter::Ended()) {
               if (cCutter::Error())
-                 Interface->Error(tr("Editing process failed!"));
+                 Skins.Message(mtError, tr("Editing process failed!"));
               else
-                 Interface->Info(tr("Editing process finished"));
+                 Skins.Message(mtInfo, tr("Editing process finished"));
               }
            }
         if (!Interact && ((!cRecordControls::Active() && !cCutter::Active() && (!Interface->HasSVDRPConnection() || UserShutdown)) || ForceShutdown)) {
@@ -755,14 +890,16 @@ int main(int argc, char *argv[])
                  if (!Next || Delta > Setup.MinEventTimeout * 60 || ForceShutdown) {
                     ForceShutdown = false;
                     if (timer)
-                       dsyslog("next timer event at %s", ctime(&Next));
+                       dsyslog("next timer event at %s", *TimeToString(Next));
                     if (WatchdogTimeout > 0)
                        signal(SIGALRM, SIG_IGN);
                     if (Interface->Confirm(tr("Press any key to cancel shutdown"), UserShutdown ? 5 : SHUTDOWNWAIT, true)) {
+                       cControl::Shutdown();
                        int Channel = timer ? timer->Channel()->Number() : 0;
                        const char *File = timer ? timer->File() : "";
+                       Delta = Next - time(NULL); // compensates for Confirm() timeout
                        char *cmd;
-                       asprintf(&cmd, "%s %ld %ld %d \"%s\" %d", Shutdown, Next, Delta, Channel, strescape(File, "\"$"), UserShutdown);
+                       asprintf(&cmd, "%s %ld %ld %d \"%s\" %d", Shutdown, Next, Delta, Channel, *strescape(File, "\"$"), UserShutdown);
                        isyslog("executing '%s'", cmd);
                        SystemExec(cmd);
                        free(cmd);
@@ -779,6 +916,7 @@ int main(int argc, char *argv[])
                  }
               // Disk housekeeping:
               RemoveDeletedRecordings();
+              cSchedules::Cleanup();
               // Plugins housekeeping:
               PluginManager.Housekeeping();
               }
@@ -786,19 +924,27 @@ int main(int argc, char *argv[])
         }
   if (Interrupted)
      isyslog("caught signal %d", Interrupted);
+
+Exit:
+
+  PluginManager.StopPlugins();
   cRecordControls::Shutdown();
   cCutter::Stop();
   delete Menu;
   cControl::Shutdown();
   delete Interface;
-  cOsd::Shutdown();
+  cOsdProvider::Shutdown();
   Remotes.Clear();
   Audios.Clear();
-  Setup.CurrentChannel = cDevice::CurrentChannel();
-  Setup.CurrentVolume  = cDevice::CurrentVolume();
-  Setup.Save();
+  Skins.Clear();
+  if (ExitCode != 2) {
+     Setup.CurrentChannel = cDevice::CurrentChannel();
+     Setup.CurrentVolume  = cDevice::CurrentVolume();
+     Setup.Save();
+     }
   cDevice::Shutdown();
-  PluginManager.Shutdown(true);
+  PluginManager.Shutdown();
+  ReportEpgBugFixStats();
   if (WatchdogTimeout > 0)
      dsyslog("max. latency time %d seconds", MaxLatencyTime);
   isyslog("exiting");
@@ -810,5 +956,5 @@ int main(int argc, char *argv[])
      esyslog("emergency exit!");
      return 1;
      }
-  return 0;
+  return ExitCode;
 }
